@@ -12,10 +12,13 @@
 
 NTL_OPEN_NNS
 
+class GF2EXModulus; // forward declaration
 
 class GF2EX {
-
 public:
+typedef GF2E coeff_type;
+typedef GF2EXModulus modulus_type;
+
 
 vec_GF2E rep;
 
@@ -29,6 +32,10 @@ vec_GF2E rep;
 
 GF2EX() { }
 
+explicit GF2EX(long a) { *this = a; }
+explicit GF2EX(GF2 a) { *this = a; }
+explicit GF2EX(const GF2& a) { *this = a; }
+
 
 GF2EX(INIT_SIZE_TYPE, long n) { rep.SetMaxLength(n); }
 
@@ -55,6 +62,15 @@ void kill()
 
    { rep.kill(); }
 
+
+
+void SetLength(long n) { rep.SetLength(n); }
+GF2E& operator[](long i) { return rep[i]; }
+const GF2E& operator[](long i) const { return rep[i]; }
+
+
+
+
 static const GF2EX& zero();
 
 
@@ -67,6 +83,13 @@ inline GF2EX(long i, long a);
 inline GF2EX(long i, GF2 a);
 inline GF2EX(long i, const GF2E& a);
 
+
+inline GF2EX(INIT_MONO_TYPE, long i, long a);
+inline GF2EX(INIT_MONO_TYPE, long i, GF2 a);
+inline GF2EX(INIT_MONO_TYPE, long i, const GF2E& a);
+inline GF2EX(INIT_MONO_TYPE, long i);
+
+
 GF2EX(GF2EX& x, INIT_TRANS_TYPE) : rep(x.rep, INIT_TRANS) { }
 
 
@@ -114,12 +137,19 @@ void SetCoeff(GF2EX& x, long i, GF2 a);
 void SetCoeff(GF2EX& x, long i, long a);
 // x[i] = a, error is raised if i < 0
 
+void SetCoeff(GF2EX& x, long i);
+// x[i] = 1, error is raised if i < 0
+
 inline GF2EX::GF2EX(long i, const GF2E& a) { SetCoeff(*this, i, a); }
 inline GF2EX::GF2EX(long i, GF2 a) { SetCoeff(*this, i, a); }
 inline GF2EX::GF2EX(long i, long a) { SetCoeff(*this, i, a); }
 
-void SetCoeff(GF2EX& x, long i);
-// x[i] = 1, error is raised if i < 0
+
+inline GF2EX::GF2EX(INIT_MONO_TYPE, long i, const GF2E& a) { SetCoeff(*this, i, a); }
+inline GF2EX::GF2EX(INIT_MONO_TYPE, long i, GF2 a) { SetCoeff(*this, i, a); }
+inline GF2EX::GF2EX(INIT_MONO_TYPE, long i, long a) { SetCoeff(*this, i, a); }
+inline GF2EX::GF2EX(INIT_MONO_TYPE, long i) { SetCoeff(*this, i); }
+
 
 void SetX(GF2EX& x);
 // x is set to the monomial X
@@ -241,7 +271,7 @@ inline GF2EX to_GF2EX(const ZZ& a)
    { GF2EX x; conv(x, a); NTL_OPT_RETURN(GF2EX, x); }
 
 #ifndef NTL_TRANSITION
-inline GF2EX to_GF2EX(GF2X& a)
+inline GF2EX to_GF2EX(const GF2X& a)
    { GF2EX x; conv(x, a); NTL_OPT_RETURN(GF2EX, x); }
 #endif
 
@@ -255,6 +285,24 @@ inline GF2EX& GF2EX::operator=(long a) { conv(*this, a); return *this; }
 
 
 
+/* additional legacy conversions for v6 conversion regime */
+
+inline void conv(GF2EX& x, const GF2EX& a)
+   { x = a; }
+
+inline void conv(vec_GF2E& x, const GF2EX& a)
+   { x = a.rep; }
+
+class ZZX;
+void conv(GF2EX& x, const ZZX& a);
+
+
+/* ------------------------------------- */
+
+
+
+
+
 /*************************************************************
 
                         Comparison
@@ -700,13 +748,7 @@ inline GF2EX& operator/=(GF2EX& x, const GF2EXModulus& F)
 
 
 
-NTL_vector_decl(GF2EX,vec_GF2EX)
-
-NTL_eq_vector_decl(GF2EX,vec_GF2EX)
-
-NTL_io_vector_decl(GF2EX,vec_GF2EX)
-
-
+typedef Vec<GF2EX> vec_GF2EX;
 
 
 
@@ -803,6 +845,7 @@ struct GF2EXArgument {
    vec_GF2EX H;
 };
 
+NTL_THREAD_LOCAL
 extern long GF2EXArgBound;
 
 
