@@ -9,9 +9,15 @@
 
 NTL_OPEN_NNS
 
+class GF2E; // forward declaration
+class GF2XModulus;
 
 class GF2X {
 public:
+typedef GF2 coeff_type;
+typedef GF2E residue_type;
+typedef GF2XModulus modulus_type;
+
 
 WordVector xrep;
 
@@ -19,6 +25,10 @@ typedef vec_GF2 VectorBaseType;
 
 
 GF2X() { }
+
+explicit GF2X(long a) { *this = a; }
+explicit GF2X(GF2 a) { *this = a; }
+
 ~GF2X() { }
 
 GF2X(INIT_SIZE_TYPE, long n);
@@ -36,12 +46,33 @@ void kill() { xrep.kill(); }
 
 void SetMaxLength(long n);
 
-static long HexOutput;
+
+
+void SetLength(long n);
+ref_GF2 operator[](long i);
+const GF2 operator[](long i) const;
+
+
+
+
+NTL_THREAD_LOCAL static long HexOutput;
 
 inline GF2X(long i, GF2 c);
 inline GF2X(long i, long c);
 
+inline GF2X(INIT_MONO_TYPE, long i, GF2 c);
+inline GF2X(INIT_MONO_TYPE, long i, long c);
+inline GF2X(INIT_MONO_TYPE, long i);
+
 GF2X(GF2X& x, INIT_TRANS_TYPE) : xrep(x.xrep, INIT_TRANS) { }
+// This should only be used for simple, local variables
+// that are not be subject to special memory management.
+
+
+
+// mainly for internal consumption by GF2XWatcher
+
+void release() { xrep.release(); }
 
 };
 
@@ -52,11 +83,11 @@ long IsOne(const GF2X& a);
 
 long IsX(const GF2X& a);
 
-GF2 coeff(const GF2X& a, long i);
+const GF2 coeff(const GF2X& a, long i);
 
-GF2 LeadCoeff(const GF2X& a);
+const GF2 LeadCoeff(const GF2X& a);
 
-GF2 ConstTerm(const GF2X& a);
+const GF2 ConstTerm(const GF2X& a);
 
 
 inline void clear(GF2X& x) 
@@ -71,11 +102,13 @@ void SetCoeff(GF2X& x, long i);
 void SetCoeff(GF2X& x, long i, GF2 a);
 void SetCoeff(GF2X& x, long i, long a);
 
-inline GF2X::GF2X(long i, GF2 a)
-   { SetCoeff(*this, i, a); }
+inline GF2X::GF2X(long i, GF2 a) { SetCoeff(*this, i, a); }
+inline GF2X::GF2X(long i, long a) { SetCoeff(*this, i, a); }
+
+inline GF2X::GF2X(INIT_MONO_TYPE, long i, GF2 a) { SetCoeff(*this, i, a); }
+inline GF2X::GF2X(INIT_MONO_TYPE, long i, long a) { SetCoeff(*this, i, a); }
+inline GF2X::GF2X(INIT_MONO_TYPE, long i) { SetCoeff(*this, i); }
 
-inline GF2X::GF2X(long i, long a)
-   { SetCoeff(*this, i, a); }
 
 void swap(GF2X& a, GF2X& b);
 
@@ -243,12 +276,7 @@ inline GF2X power(const GF2X& a, long e)
 
 
 
-NTL_vector_decl(GF2X,vec_GF2X)
-
-NTL_io_vector_decl(GF2X,vec_GF2X)
-
-NTL_eq_vector_decl(GF2X,vec_GF2X)
-
+typedef Vec<GF2X> vec_GF2X;
 
 void LeftShift(GF2X& c, const GF2X& a, long n);
 inline GF2X LeftShift(const GF2X& a, long n)
@@ -515,6 +543,24 @@ inline GF2X to_GF2X(const ZZ& a)
 inline vec_GF2 to_vec_GF2(const GF2X& a)
    { vec_GF2 x; conv(x, a); NTL_OPT_RETURN(vec_GF2, x); }
 
+
+
+/* additional legacy conversions for v6 conversion regime */
+
+inline void conv(GF2X& x, const GF2X& a)
+   { x = a; }
+
+class ZZX;
+void conv(GF2X& x, const ZZX& a);
+void conv(ZZX& x, const GF2X& a);
+
+
+/* ------------------------------------- */
+
+
+
+
+
 inline GF2X& GF2X::operator=(long a)
    { conv(*this, a); return *this; }
 
@@ -619,7 +665,7 @@ inline vec_GF2 UpdateMap(const vec_GF2& a,
                        const GF2XTransMultiplier& B, const GF2XModulus& F)
    { vec_GF2 x; UpdateMap(x, a, B, F); NTL_OPT_RETURN(vec_GF2, x); }
 
-inline void project(GF2& x, const vec_GF2& a, const GF2X& b)
+inline void project(ref_GF2 x, const vec_GF2& a, const GF2X& b)
    { x = to_GF2(InnerProduct(a.rep, b.xrep)); }
 
 inline GF2 project(const vec_GF2& a, const GF2X& b)
@@ -648,12 +694,12 @@ inline vec_GF2 TraceVec(const GF2X& f)
    { vec_GF2 x; TraceVec(x, f); NTL_OPT_RETURN(vec_GF2, x); }
 
 
-void TraceMod(GF2& x, const GF2X& a, const GF2XModulus& F);
+void TraceMod(ref_GF2 x, const GF2X& a, const GF2XModulus& F);
 
 inline GF2 TraceMod(const GF2X& a, const GF2XModulus& F)
    { GF2 x; TraceMod(x, a, F); return x; }
 
-void TraceMod(GF2& x, const GF2X& a, const GF2X& f);
+void TraceMod(ref_GF2 x, const GF2X& a, const GF2X& f);
 
 inline GF2 TraceMod(const GF2X& a, const GF2X& f)
    { GF2 x; TraceMod(x, a, f); return x; }
@@ -673,6 +719,24 @@ inline long NumBytes(const GF2X& a)
    { return (NumBits(a) + 7)/8; }
 
 
+
+// GF2X scratch variabes
+
+
+class GF2XWatcher {
+public:
+   GF2X *watched;
+   explicit
+   GF2XWatcher(GF2X *_watched) : watched(_watched) {}
+
+   ~GF2XWatcher() { watched->release(); } 
+};
+
+#define NTL_GF2XRegister(x) NTL_THREAD_LOCAL static GF2X x; GF2XWatcher _WATCHER__ ## x(&x)
+
+
+
+
 NTL_CLOSE_NNS
 
 #endif
