@@ -17,6 +17,8 @@ long e;
 
 RR() {  e = 0; }
 
+explicit RR(double a) : e(0) { *this = a; }
+
 inline RR(INIT_VAL_TYPE, const ZZ& a);
 inline RR(INIT_VAL_TYPE, int a);
 inline RR(INIT_VAL_TYPE, long a);
@@ -43,11 +45,11 @@ RR(RR& z, INIT_TRANS_TYPE) : x(z.x, INIT_TRANS), e(z.e) { }
 const ZZ& mantissa() const { return x; }
 long exponent() const { return e; }
 
-static long prec;
+NTL_THREAD_LOCAL static long prec;
 static void SetPrecision(long p);
 static long precision() { return prec; }
 
-static long oprec;
+NTL_THREAD_LOCAL static long oprec;
 static void SetOutputPrecision(long p);
 static long OutputPrecision() { return oprec; }
 
@@ -424,6 +426,18 @@ inline void FloorToZZ(ZZ& z, const RR& a) { conv(z, a); }
 inline ZZ FloorToZZ(const RR& a)
    { ZZ z; conv(z, a); NTL_OPT_RETURN(ZZ, z); }
 
+
+/* additional legacy conversions for v6 conversion regime */
+
+inline void conv(unsigned int& x, const RR& a)
+   { long z; conv(z, a); conv(x, z); }
+
+inline void conv(unsigned long& x, const RR& a)
+   { long z; conv(z, a); conv(x, z); }
+
+
+/* ------------------------------------- */
+
 void MakeRR(RR& z, const ZZ& a,  long e);
 inline RR MakeRR(const ZZ& a,  long e)
    { RR z; MakeRR(z, a, e); NTL_OPT_RETURN(RR, z); }
