@@ -17,26 +17,29 @@
 
 #include <NTL/lip.h>
 #include <NTL/tools.h>
+#include <NTL/vector.h>
 
 NTL_OPEN_NNS
 
 
-
+class ZZ_p; // forward declaration
+class ZZX;
 
 class ZZ {
 public:
+typedef ZZ_p residue_type;
+typedef ZZX poly_type;
 
 NTL_verylong rep; // This is currently public for "emergency" situations
                    // May be private in future versions.
 
 
-ZZ() 
+ZZ() : rep(0) { }
 // initial value is 0.
 
-{ rep = 0; }
+explicit ZZ(long a) : rep(0) { *this = a; }
 
-
-ZZ(INIT_SIZE_TYPE, long k)
+ZZ(INIT_SIZE_TYPE, long k) : rep(0)
 // initial value is 0, but space is pre-allocated so that numbers
 // x with x.size() <= k can be stored without re-allocation.
 // Call with ZZ(INIT_SIZE, k).
@@ -45,24 +48,22 @@ ZZ(INIT_SIZE_TYPE, long k)
 
 
 {
-   rep = 0;
    NTL_zsetlength(&rep, k); 
 }
 
-ZZ(const ZZ& a)
+ZZ(const ZZ& a) : rep(0)
 // initial value is a.
 
 {
-   rep = 0;
    NTL_zcopy(a.rep, &rep);
 }
 
 
-ZZ(INIT_VAL_TYPE, long a) { rep = 0; NTL_zintoz(a, &rep); }
-ZZ(INIT_VAL_TYPE, int a) { rep = 0; NTL_zintoz(a, &rep); }
+ZZ(INIT_VAL_TYPE, long a) : rep(0) { NTL_zintoz(a, &rep); }
+ZZ(INIT_VAL_TYPE, int a) : rep(0) { NTL_zintoz(a, &rep); }
 
-ZZ(INIT_VAL_TYPE, unsigned long a) { rep = 0; NTL_zuintoz(a, &rep); }
-ZZ(INIT_VAL_TYPE, unsigned int a) { rep = 0; NTL_zuintoz((unsigned long) a, &rep); }
+ZZ(INIT_VAL_TYPE, unsigned long a) : rep(0) { NTL_zuintoz(a, &rep); }
+ZZ(INIT_VAL_TYPE, unsigned int a) : rep(0) { NTL_zuintoz((unsigned long) a, &rep); }
 
 inline ZZ(INIT_VAL_TYPE, const char *);
 inline ZZ(INIT_VAL_TYPE, float);
@@ -89,9 +90,16 @@ void SetSize(long k)
 { NTL_zsetlength(&rep, k); }
 
 long size() const
+// returns the number of (NTL_ZZ_NBIT-bit) digits of |a|; the size of 0 is 0.
    { return NTL_zsize(rep); }
 
-// returns the number of (NTL_ZZ_NBIT-bit) digits of |a|; the size of 0 is 0.
+long null() const
+// test of rep is null
+   { return !rep; }
+
+long MaxAlloc() const
+// returns max allocation request, possibly rounded up a bit...
+   { return NTL_zmaxalloc(rep); }
 
 
 long SinglePrecision() const
@@ -110,11 +118,34 @@ static const ZZ& zero();
 ZZ(ZZ& x, INIT_TRANS_TYPE) { rep = x.rep; x.rep = 0; }
 // used to cheaply hand off memory management of return value,
 // without copying, assuming compiler implements the
-// "return value optimization"
+// "return value optimization".  This is probably obsolete by
+// now, as modern compilers can and should optimize
+// the copy constructor in the situations where this is used.
+// This should only be used for simple, local variables
+// that are not be subject to special memory management.
+
+
+// mainly for internal consumption by ZZWatcher
+
+void release() { if (MaxAlloc() > NTL_RELEASE_THRESH) kill(); }
 
 };
 
 
+class ZZWatcher {
+public:
+   ZZ *watched;
+   explicit
+   ZZWatcher(ZZ *_watched) : watched(_watched) {}
+
+   ~ZZWatcher() { watched->release(); }
+};
+
+#define NTL_ZZRegister(x) NTL_THREAD_LOCAL static ZZ x; ZZWatcher _WATCHER__ ## x(&x)
+
+
+
+
 
 const ZZ& ZZ_expo(long e);
 
@@ -166,15 +197,15 @@ inline void conv(ZZ& x, unsigned int a) { NTL_zuintoz((unsigned long)(a), &x.rep
 inline ZZ to_ZZ(unsigned int a) { return ZZ(INIT_VAL, a); }
 
 void conv(ZZ& x, const char *s);
-inline ZZ::ZZ(INIT_VAL_TYPE, const char *s) {  rep = 0; conv(*this, s); }
+inline ZZ::ZZ(INIT_VAL_TYPE, const char *s) : rep(0) { conv(*this, s); }
 inline ZZ to_ZZ(const char *s) { return ZZ(INIT_VAL, s); }
 
 inline void conv(ZZ& x, double a) { NTL_zdoubtoz(a, &x.rep); }
-inline ZZ::ZZ(INIT_VAL_TYPE, double a) { rep = 0; conv(*this, a); }
+inline ZZ::ZZ(INIT_VAL_TYPE, double a) : rep(0) { conv(*this, a); }
 inline ZZ to_ZZ(double a) { return ZZ(INIT_VAL, a); }
 
 inline void conv(ZZ& x, float a) { NTL_zdoubtoz(double(a), &x.rep); }
-inline ZZ::ZZ(INIT_VAL_TYPE, float a) { rep = 0; conv(*this, a); }
+inline ZZ::ZZ(INIT_VAL_TYPE, float a) : rep(0) { conv(*this, a); }
 inline ZZ to_ZZ(float a) { return ZZ(INIT_VAL, a); }
 
 inline void conv(long& x, const ZZ& a) { x = NTL_ztoint(a.rep); }
@@ -1193,9 +1224,8 @@ inline ZZ SqrRootMod(const ZZ& a, const ZZ& n)
 
 class PrimeSeq {
 
-
 char *movesieve;
-char *movesieve_mem;
+Vec<char> movesieve_mem;
 long pindex;
 long pshift;
 long exhausted;
@@ -1203,7 +1233,6 @@ long exhausted;
 public:
 
 PrimeSeq();
-~PrimeSeq();
 
 long next();
 // returns next prime in the sequence.
@@ -1259,6 +1288,13 @@ The number n itself should be in the range 1..2^{NTL_SP_NBITS}-1.
 // are inconsistent across severeal files.
 // Maybe an unnamed namespace would be better.
 
+// DIRT: undocumented feature: in all of these MulMod routines,
+// the first argument, a, need only be in the range
+// 0..2^{NTL_SP_NBITS}-1.  This is assumption is used internally
+// in some NT routines...I've tried to mark all such uses with a
+// DIRT comment.  I may decide to make this feature part
+// of the documented interface at some point in the future.
+
 
 
 
@@ -1319,216 +1355,7 @@ static inline long NegateMod(long a, long n)
 #endif
 
 
-#if (defined(NTL_SINGLE_MUL))
-
-
-#if (!defined(NTL_FAST_INT_MUL))
-
-
-static inline long MulMod(long a, long b, long n)
-// return (a*b)%n
-
-{
-   double ab;
-   long q, res;
-
-   ab = ((double) a) * ((double) b);
-   q  = (long) (ab/((double) n));  // q could be off by (+/-) 1
-   res = (long) (ab - ((double) q)*((double) n));
-#if (NTL_ARITH_RIGHT_SHIFT && defined(NTL_AVOID_BRANCHING))
-   res += (res >> (NTL_BITS_PER_LONG-1)) & n;
-   res -= n;
-   res += (res >> (NTL_BITS_PER_LONG-1)) & n;
-#else
-   if (res >= n)
-      res -= n;
-   else if (res < 0)
-      res += n;
-#endif
-   return res;
-}
-
-/*
-The following MulMod takes a fourth argument, ninv,
-which is assumed to equal 1/((double) n).
-It is usually faster than the above.
-*/
-
-static inline long MulMod(long a, long b, long n, double ninv)
-{
-   double ab;
-   long q, res;
-
-   ab = ((double) a) * ((double) b);
-   q  = (long) (ab*ninv);   // q could be off by (+/-) 1
-   res = (long) (ab - ((double) q)*((double) n));
-#if (NTL_ARITH_RIGHT_SHIFT && defined(NTL_AVOID_BRANCHING))
-   res += (res >> (NTL_BITS_PER_LONG-1)) & n;
-   res -= n;
-   res += (res >> (NTL_BITS_PER_LONG-1)) & n;
-#else
-   if (res >= n)
-      res -= n;
-   else if (res < 0)
-      res += n;
-#endif
-   return res;
-}
-
-/*
-Yet another MulMod.
-This time, the 4th argument should be ((double) b)/((double) n).
-*/
-
-static inline long MulMod2(long a, long b, long n, double bninv)
-{
-   double ab;
-   long q, res;
-
-   ab = ((double) a)*((double) b);
-   q = (long) (((double) a)*bninv);
-   res = (long) (ab - ((double) q)*((double) n));
-#if (NTL_ARITH_RIGHT_SHIFT && defined(NTL_AVOID_BRANCHING))
-   res += (res >> (NTL_BITS_PER_LONG-1)) & n;
-   res -= n;
-   res += (res >> (NTL_BITS_PER_LONG-1)) & n;
-#else
-   if (res >= n)
-      res -= n;
-   else if (res < 0)
-      res += n;
-#endif
-   return res;
-}
-
-static inline long MulDivRem(long& qq, long a, long b, long n, double bninv)
-{
-   double ab;
-   long q, res;
-
-   ab = ((double) a)*((double) b);
-   q = (long) (((double) a)*bninv);
-   res = (long) (ab - ((double) q)*((double) n));
-   if (res >= n) {
-      res -= n;
-      q++;
-   } else if (res < 0) {
-      res += n;
-      q--;
-   }
-
-   qq = q;
-   return res;
-}
-
-#else
-
-static inline long MulMod(long a, long b, long n)
-// return (a*b)%n
-
-{
-   double ab, xx;
-   long iab, q, res;
-
-   ab = ((double) a) * ((double) b);
-   q  = (long) (ab/((double) n));  // q could be off by (+/-) 1
-
-   xx = ab + 4503599627370496.0;
-   NTL_FetchLo(iab, xx);
-
-   res = iab - q*n;
-
-#if (NTL_ARITH_RIGHT_SHIFT && defined(NTL_AVOID_BRANCHING))
-   res += (res >> (NTL_BITS_PER_LONG-1)) & n;
-   res -= n;
-   res += (res >> (NTL_BITS_PER_LONG-1)) & n;
-#else
-   if (res >= n)
-      res -= n;
-   else if (res < 0)
-      res += n;
-#endif
-   return res;
-}
-
-/*
-The following MulMod takes a fourth argument, ninv,
-which is assumed to equal 1/((double) n).
-It is usually faster than the above.
-*/
-
-static inline long MulMod(long a, long b, long n, double ninv)
-{
-   double ab, xx;
-   long iab, q, res;
-
-   ab = ((double) a) * ((double) b);
-   q  = (long) (ab*ninv);   // q could be off by (+/-) 1
-
-   xx = ab + 4503599627370496.0;
-   NTL_FetchLo(iab, xx);
-
-   res = iab - q*n;
-#if (NTL_ARITH_RIGHT_SHIFT && defined(NTL_AVOID_BRANCHING))
-   res += (res >> (NTL_BITS_PER_LONG-1)) & n;
-   res -= n;
-   res += (res >> (NTL_BITS_PER_LONG-1)) & n;
-#else
-   if (res >= n)
-      res -= n;
-   else if (res < 0)
-      res += n;
-#endif
-   return res;
-}
-
-/*
-Yet another MulMod.
-This time, the 4th argument should be ((double) b)/((double) n).
-*/
-
-static inline long MulMod2(long a, long b, long n, double bninv)
-{
-   long q, res;
-
-   q = (long) (((double) a)*bninv);
-   res = a*b - q*n;
-#if (NTL_ARITH_RIGHT_SHIFT && defined(NTL_AVOID_BRANCHING))
-   res += (res >> (NTL_BITS_PER_LONG-1)) & n;
-   res -= n;
-   res += (res >> (NTL_BITS_PER_LONG-1)) & n;
-#else
-   if (res >= n)
-      res -= n;
-   else if (res < 0)
-      res += n;
-#endif
-   return res;
-}
-
-
-static inline long MulDivRem(long& qq, long a, long b, long n, double bninv)
-{
-   long q, res;
-
-   q = (long) (((double) a)*bninv);
-   res = a*b - q*n;
-   if (res >= n) {
-      res -= n;
-      q++;
-   } else if (res < 0) {
-      res += n;
-      q--;
-   }
-
-   qq = q;
-   return res;
-}
-
-#endif
-
-
-#elif (!defined(NTL_CLEAN_SPMM))
+#if (!defined(NTL_CLEAN_SPMM))
 
 
 /*
@@ -1723,10 +1550,10 @@ static inline long MulDivRem(long& qq, long a, long b, long n, double bninv)
 // significantly faster.  There are four possible implementations:
 //  - default: uses MulMod2 above (lots of floating point)
 //  - NTL_SPMM_ULL: uses unsigned long long (if possible)
-//  - NTL_SMPP_ASM: uses assembly language (if possible)
+//  - NTL_SPMM_ASM: uses assembly language (if possible)
 //  - NTL_SPMM_UL: uses only unsigned long arithmetic (portable, slower).
 
-#if (!defined(NTL_SINGLE_MUL) && (defined(NTL_SPMM_ULL) || defined(NTL_SPMM_ASM)))
+#if ((defined(NTL_SPMM_ULL) || defined(NTL_SPMM_ASM)))
 
 
 // unsigned long long / asm versions
@@ -1860,7 +1687,7 @@ static inline long MulModPrecon(long a, long b, long n, unsigned long bninv)
 
 
 
-#elif (!defined(NTL_SINGLE_MUL) && defined(NTL_SPMM_UL))
+#elif (defined(NTL_SPMM_UL))
 
 // plain, portable (but slower) int version
 
@@ -2015,6 +1842,32 @@ long PowerMod(long a, long e, long n);
 // computes a^e mod n, e >= 0
 
 
+inline
+void VectorMulModPrecon(long k, long *x, const long *a, long b, long n, 
+                        mulmod_precon_t bninv)
+{
+   for (long i = 0; i < k; i++)
+      x[i] = MulModPrecon(a[i], b, n, bninv);
+}
+
+inline
+void VectorMulMod(long k, long *x, const long *a, long b, long n, 
+                  double ninv)
+{
+   mulmod_precon_t bninv;
+   bninv = PrepMulModPrecon(b, n, ninv);
+   VectorMulModPrecon(k, x, a, b, n, bninv);
+}
+
+
+inline 
+void VectorMulMod(long k, long *x, const long *a, long b, long n)
+{
+   double ninv = 1/((double) n);
+   VectorMulMod(k, x, a, b, n, ninv);
+}
+
+
 NTL_CLOSE_NNS
 
 
