@@ -2,35 +2,13 @@
 
 typedef long * _ntl_verylong;
 
-#if (defined(NTL_SINGLE_MUL))
-
-#if (defined(NTL_AVOID_FLOAT) || defined(NTL_LONG_LONG))
-#error "at most one of NTL_SINGLE_MUL NTL_AVOID_FLOAT NTL_LONG_LONG allowed"
+#if (defined(NTL_AVOID_FLOAT) && defined(NTL_LONG_LONG))
+#error "at most one of NTL_AVOID_FLOAT NTL_LONG_LONG allowed"
 #endif
 
-#elif (defined(NTL_AVOID_FLOAT) && defined(NTL_LONG_LONG))
-#error "at most one of NTL_SINGLE_MUL NTL_AVOID_FLOAT NTL_LONG_LONG allowed"
-#endif
-
-#if (defined(NTL_SINGLE_MUL))
-
-#if (!NTL_SINGLE_MUL_OK)
-#error "NTL_SINGLE_MUL not supported on this platform"
-#endif
-
-#if (defined(NTL_CLEAN_INT))
-#error "NTL_SINGLE_MUL not allowed with NTL_CLEAN_INT"
-#endif
-
-
-#define NTL_NBITS (26)
-
-#else
 
 #define NTL_NBITS NTL_NBITS_MAX
 
-#endif
-
 
 #define NTL_RADIX           (1L<<NTL_NBITS)
 #define NTL_NBITSH          (NTL_NBITS>>1)
@@ -55,49 +33,6 @@ typedef long * _ntl_verylong;
 
 
 
-#if (defined(NTL_SINGLE_MUL) && !NTL_SINGLE_MUL_OK)
-#undef NTL_SINGLE_MUL
-#endif
-
-#if (defined(NTL_SINGLE_MUL))
-
-
-/****************************************************************
-
-The following macros extract the two words of a double,
-getting around the type system.
-This is only used in the NTL_SINGLE_MUL strategy.
-
-*****************************************************************/
-
-#if (NTL_DOUBLES_LOW_HIGH)
-#define NTL_LO_WD 0
-#define NTL_HI_WD 1
-#else
-#define NTL_LO_WD 1
-#define NTL_HI_WD 0
-#endif
-
-
-typedef union { double d; unsigned long rep[2]; } _ntl_d_or_rep;
-
-#define NTL_FetchHiLo(hi,lo,x) \
-do { \
-   _ntl_d_or_rep ll_xx; \
-   ll_xx.d = (x); \
-   hi = ll_xx.rep[NTL_HI_WD]; \
-   lo = ll_xx.rep[NTL_LO_WD]; \
-} while (0)
-
-
-#define NTL_FetchLo(lo,x)  \
-do {  \
-   _ntl_d_or_rep ll_xx;  \
-   ll_xx.d = x;  \
-   lo = ll_xx.rep[NTL_LO_WD];  \
-} while (0) 
-
-#endif
 
 
 /**********************************************************************
@@ -123,10 +58,6 @@ do {  \
 
 
 
-#if (defined(__cplusplus) && !defined(NTL_CXX_ONLY))
-extern "C" {
-#endif
-
 
 /***********************************************************************
 
@@ -511,6 +442,10 @@ extern "C" {
 ***********************************************************************/
 
 
+    long _ntl_zmaxalloc(_ntl_verylong x); 
+      /* max allocation request, possibly rounded up a bit */
+
+
     void _ntl_zsetlength(_ntl_verylong *v, long len);
        /* Allocates enough space to hold a len-digit number,
           where each digit has NTL_NBITS bits.
@@ -562,13 +497,8 @@ void _ntl_rem_struct_eval(void *rem_struct, long *x, _ntl_verylong a);
 
 
 
-#if (defined(__cplusplus) && !defined(NTL_CXX_ONLY))
-}
-#endif
 
 
-extern int _ntl_gmp_hack;
-
 #define NTL_crt_struct_eval _ntl_crt_struct_eval
 #define NTL_crt_struct_free _ntl_crt_struct_free
 #define NTL_crt_struct_init _ntl_crt_struct_init
@@ -627,6 +557,7 @@ extern int _ntl_gmp_hack;
 #define NTL_zscompare _ntl_zscompare
 #define NTL_zsdiv _ntl_zsdiv
 #define NTL_zsetbit _ntl_zsetbit
+#define NTL_zmaxalloc _ntl_zmaxalloc
 #define NTL_zsetlength _ntl_zsetlength
 #define NTL_zsign _ntl_zsign
 #define NTL_zsize _ntl_zsize
