@@ -242,9 +242,20 @@
  */
 
 
-#if (defined(__cplusplus) && !defined(NTL_CXX_ONLY))
-extern "C" {
-#endif
+#define NTL_THREAD_LOCAL 
+
+/* 
+ * placeholder for now...it will eventually be defined as thread_local
+ */
+
+
+#define NTL_RELEASE_THRESH (128)
+
+/*
+ * threshold for releasing scratch memory.
+ */
+
+
 
 long _ntl_IsFinite(double *p);
 /* This forces a double into memory, and tests if it is "normal";
@@ -258,10 +269,20 @@ void _ntl_ForceToMem(double *p);
    a double into memory (see comment above). */
 
 double _ntl_ldexp(double x, long e);
+
+void _ntl_abort();
+/* This is the routine called by NTL to abort a program in case of error. */
+
+void _ntl_abort_cxx_callback();
+/* This is a C++ function (implemented in tools.c) that is
+   used to implement the callback mechanism.  The issue here
+   is that I don't want a C function to call a C++ function
+   via a function pointer.  This could potentially be problematic. 
+   EDIT: since changing over to all-C++, this is now moot.
+*/
+
+   
    
-#if (defined(__cplusplus) && !defined(NTL_CXX_ONLY))
-}
-#endif
 
 #endif
 
