@@ -6,9 +6,13 @@
 
 NTL_OPEN_NNS
 
-class zz_pEX {
+class zz_pEXModulus; // forward declaration
 
+class zz_pEX {
 public:
+typedef zz_pE coeff_type;
+typedef zz_pEXModulus modulus_type;
+
 
 vec_zz_pE rep;
 
@@ -20,11 +24,13 @@ vec_zz_pE rep;
 ****************************************************************/
 
 
-zz_pEX()
+zz_pEX() { }
 //  initial value 0
 
-   { }
 
+explicit zz_pEX(long a) { *this = a; }
+explicit zz_pEX(const zz_p& a) { *this = a; }
+explicit zz_pEX(const zz_pE& a) { *this = a; }
 
 zz_pEX(INIT_SIZE_TYPE, long n) { rep.SetMaxLength(n); }
 
@@ -45,12 +51,26 @@ void kill()
 
    { rep.kill(); }
 
+
+
+void SetLength(long n) { rep.SetLength(n); }
+zz_pE& operator[](long i) { return rep[i]; }
+const zz_pE& operator[](long i) const { return rep[i]; }
+
+
+
+
 static const zz_pEX& zero();
 
 inline zz_pEX(long i, const zz_pE& c);
 inline zz_pEX(long i, const zz_p& c);
 inline zz_pEX(long i, long c);
 
+inline zz_pEX(INIT_MONO_TYPE, long i, const zz_pE& c);
+inline zz_pEX(INIT_MONO_TYPE, long i, const zz_p& c);
+inline zz_pEX(INIT_MONO_TYPE, long i, long c);
+inline zz_pEX(INIT_MONO_TYPE, long i);
+
 
 inline zz_pEX& operator=(long a);
 inline zz_pEX& operator=(const zz_p& a);
@@ -93,18 +113,18 @@ void SetCoeff(zz_pEX& x, long i, const zz_p& a);
 void SetCoeff(zz_pEX& x, long i, long a);
 // x[i] = a, error is raised if i < 0
 
-inline zz_pEX::zz_pEX(long i, const zz_pE& a)
-   { SetCoeff(*this, i, a); }
-
-inline zz_pEX::zz_pEX(long i, const zz_p& a)
-   { SetCoeff(*this, i, a); }
-
-inline zz_pEX::zz_pEX(long i, long a)
-   { SetCoeff(*this, i, a); }
-
 void SetCoeff(zz_pEX& x, long i);
 // x[i] = 1, error is raised if i < 0
 
+inline zz_pEX::zz_pEX(long i, const zz_pE& a) { SetCoeff(*this, i, a); }
+inline zz_pEX::zz_pEX(long i, const zz_p& a) { SetCoeff(*this, i, a); }
+inline zz_pEX::zz_pEX(long i, long a) { SetCoeff(*this, i, a); }
+
+inline zz_pEX::zz_pEX(INIT_MONO_TYPE, long i, const zz_pE& a) { SetCoeff(*this, i, a); }
+inline zz_pEX::zz_pEX(INIT_MONO_TYPE, long i, const zz_p& a) { SetCoeff(*this, i, a); }
+inline zz_pEX::zz_pEX(INIT_MONO_TYPE, long i, long a) { SetCoeff(*this, i, a); }
+inline zz_pEX::zz_pEX(INIT_MONO_TYPE, long i) { SetCoeff(*this, i); }
+
 void SetX(zz_pEX& x);
 // x is set to the monomial X
 
@@ -242,6 +262,24 @@ inline zz_pEX& zz_pEX::operator=(const zz_pE& a)
    { conv(*this, a); return *this; }
 
 
+
+
+/* additional legacy conversions for v6 conversion regime */
+
+inline void conv(zz_pEX& x, const zz_pEX& a)
+   { x = a; }
+
+inline void conv(vec_zz_pE& x, const zz_pEX& a)
+   { x = a.rep; }
+
+class ZZX;
+void conv(zz_pEX& x, const ZZX& a);
+
+
+/* ------------------------------------- */
+
+
+
 /*************************************************************
 
                         Comparison
@@ -705,13 +743,7 @@ inline zz_pEX& operator/=(zz_pEX& x, const zz_pEXModulus& F)
 
 
 
-NTL_vector_decl(zz_pEX,vec_zz_pEX)
-
-NTL_eq_vector_decl(zz_pEX,vec_zz_pEX)
-
-NTL_io_vector_decl(zz_pEX,vec_zz_pEX)
-
-
+typedef Vec<zz_pEX> vec_zz_pEX;
 
 
 
@@ -807,7 +839,7 @@ struct zz_pEXArgument {
    vec_zz_pEX H;
 };
 
-extern long zz_pEXArgBound;
+NTL_THREAD_LOCAL extern long zz_pEXArgBound;
 
 
 void build(zz_pEXArgument& H, const zz_pEX& h, const zz_pEXModulus& F, long m);
