@@ -99,9 +99,6 @@ float floor(float);
 
 
 
-#if (!defined(NTL_CXX_ONLY)) 
-extern "C"
-#endif
 double _ntl_GetTime();
 
 typedef unsigned long _ntl_ulong;
@@ -115,6 +112,15 @@ typedef _ntl_ulong *_ntl_ulong_ptr;
 
 NTL_OPEN_NNS
 
+
+#define NTL_FILE_THRESH (64000.0)
+// threshold in KB for switching to external storage of certain
+// tables (currently in the DDF polynomial factoring routines)
+
+
+
+NTL_THREAD_LOCAL extern void (*ErrorCallback)();
+
 struct INIT_SIZE_STRUCT { };
 const INIT_SIZE_STRUCT INIT_SIZE = INIT_SIZE_STRUCT();
 typedef const INIT_SIZE_STRUCT& INIT_SIZE_TYPE;
@@ -136,6 +142,23 @@ struct INIT_FFT_STRUCT { };
 const INIT_FFT_STRUCT INIT_FFT = INIT_FFT_STRUCT();
 typedef const INIT_FFT_STRUCT& INIT_FFT_TYPE;
 
+struct INIT_USER_FFT_STRUCT { };
+const INIT_USER_FFT_STRUCT INIT_USER_FFT = INIT_USER_FFT_STRUCT();
+typedef const INIT_USER_FFT_STRUCT& INIT_USER_FFT_TYPE;
+
+struct INIT_NO_ALLOC_STRUCT { };
+const INIT_NO_ALLOC_STRUCT INIT_NO_ALLOC = INIT_NO_ALLOC_STRUCT();
+typedef const INIT_NO_ALLOC_STRUCT& INIT_NO_ALLOC_TYPE;
+
+struct INIT_ALLOC_STRUCT { };
+const INIT_ALLOC_STRUCT INIT_ALLOC = INIT_ALLOC_STRUCT();
+typedef const INIT_ALLOC_STRUCT& INIT_ALLOC_TYPE;
+
+struct INIT_MONO_STRUCT { };
+const INIT_MONO_STRUCT INIT_MONO = INIT_MONO_STRUCT();
+typedef const INIT_MONO_STRUCT& INIT_MONO_TYPE;
+
+
 
 #ifdef NTL_NO_INIT_TRANS
 #define NTL_OPT_RETURN(t, x) return x
@@ -242,8 +265,46 @@ inline double to_double(float a) { return double(a); }
 inline double to_double(double a) { return a; }
 
 
+
+/* additional legacy conversions for v6 conversion regime */
+
+
+inline void conv(unsigned int& x, int a) { x = ((unsigned int)(a)); }
+inline void conv(unsigned int& x, long a) { x = ((unsigned int)(a)); }
+inline void conv(unsigned int& x, unsigned a) { x = a; }
+inline void conv(unsigned int& x, unsigned long a) { x = ((unsigned int)(a)); }
+inline void conv(unsigned int& x, float a) { x = ((unsigned int) to_long(a)); }
+inline void conv(unsigned int& x, double a) { x = ((unsigned int) to_long(a)); }
+
+inline void conv(unsigned long& x, int a) { x = ((unsigned long)(a)); }
+inline void conv(unsigned long& x, long a) { x = ((unsigned long)(a)); }
+inline void conv(unsigned long& x, unsigned a) { x = ((unsigned long)(a)); }
+inline void conv(unsigned long& x, unsigned long a) { x = a; }
+inline void conv(unsigned long& x, float a) { x = ((unsigned int) to_long(a)); }
+inline void conv(unsigned long& x, double a) { x = ((unsigned int) to_long(a)); }
+
+
+/* ------------------------------------- */
+
+
+// new style converson function
+//   example: ZZ x = conv<ZZ>(1);
+//   note: modern C++ compilers should implemented 
+//     "named return value optimization", so the
+//     result statement should not create a temporary
+
+template<class T, class S>
+T conv(const S& a)
+{
+   T x;
+   conv(x, a);
+   return x;
+}
+
+
 long SkipWhiteSpace(NTL_SNS istream& s);
 long IsWhiteSpace(long c);
+long IsEOFChar(long c);
 
 long CharToIntVal(long c);
 char IntValToChar(long a);
@@ -272,6 +333,21 @@ inline void ForceToMem(double *p) { }
 
 void PrintTime(NTL_SNS ostream& s, double t);
 
+
+
+#if (defined(__GNUC__) && (__GNUC__ >= 4))
+
+// on relative modern versions of gcc, we can 
+// decalare "restricted" pointers in C++
+
+#define NTL_RESTRICT __restrict
+
+#else
+
+#define NTL_RESTRICT
+
+#endif
+
 NTL_CLOSE_NNS
 
 
