@@ -6,11 +6,7 @@
 
 NTL_OPEN_NNS
 
-NTL_vector_decl(_ntl_ulong,vec_ulong)
-
-NTL_io_vector_decl(_ntl_ulong,vec_ulong)
-
-NTL_eq_vector_decl(_ntl_ulong,vec_ulong)
+typedef Vec<unsigned long> vec_ulong;
 
 NTL_CLOSE_NNS
 
