@@ -40,9 +40,6 @@ int main()
    cout << "NTL_GMP_LIP\n";
 #endif
 
-#ifdef NTL_GMP_HACK
-   cout << "NTL_GMP_HACK\n";
-#endif
 
 #ifdef NTL_GF2X_LIB
    cout << "NTL_GF2X_LIB\n";
@@ -58,10 +55,6 @@ int main()
    cout << make_string(NTL_UNSIGNED_LONG_LONG_TYPE) << "\n";
 #endif
 
-#ifdef NTL_CXX_ONLY
-   cout << "NTL_CXX_ONLY\n";
-#endif
-
 
 #ifdef NTL_X86_FIX
    cout << "NTL_X86_FIX\n";
@@ -120,12 +113,17 @@ cout << "Performance Options:\n";
 #endif
 
 
-
-
 #ifdef NTL_AVOID_BRANCHING
    cout << "NTL_AVOID_BRANCHING\n";
 #endif
 
+#ifdef NTL_FFT_BIGTAB
+   cout << "NTL_FFT_BIGTAB\n";
+#endif
+
+#ifdef NTL_FFT_LAZYMUL
+   cout << "NTL_FFT_LAZYMUL\n";
+#endif
 
 
 #ifdef NTL_TBL_REM
