@@ -2,6 +2,7 @@
 
 #include <NTL/GF2EX.h>
 #include <NTL/vec_vec_GF2.h>
+#include <NTL/ZZX.h>
 
 #include <NTL/new.h>
 
@@ -11,7 +12,7 @@ NTL_START_IMPL
 
 const GF2EX& GF2EX::zero()
 {
-   static GF2EX z;
+   NTL_THREAD_LOCAL static GF2EX z;
    return z;
 }
 
@@ -240,6 +241,27 @@ void conv(GF2EX& x, const vec_GF2E& a)
 }
 
 
+
+/* additional legacy conversions for v6 conversion regime */
+
+void conv(GF2EX& x, const ZZX& a)
+{
+   long n = a.rep.length();
+   long i;
+
+   x.rep.SetLength(n);
+   for (i = 0; i < n; i++)
+      conv(x.rep[i], a.rep[i]);
+
+   x.normalize();
+}
+
+
+/* ------------------------------------- */
+
+
+
+
 void add(GF2EX& x, const GF2EX& a, const GF2EX& b)
 {
    long da = deg(a);
@@ -1441,7 +1463,7 @@ void NewtonInvTrunc(GF2EX& c, const GF2EX& a, long e)
       return;
    }
 
-   static vec_long E;
+   vec_long E;
    E.SetLength(0);
    append(E, e);
    while (e > 1) {
@@ -2207,14 +2229,6 @@ void ShiftAdd(GF2EX& U, const GF2EX& V, long n)
    U.normalize();
 }
 
-NTL_vector_impl(GF2EX,vec_GF2EX)
-
-NTL_eq_vector_impl(GF2EX,vec_GF2EX)
-
-NTL_io_vector_impl(GF2EX,vec_GF2EX)
-
-
-
 
 void IterBuild(GF2E* a, long n)
 {
@@ -2445,6 +2459,7 @@ void build(GF2EXArgument& A, const GF2EX& h, const GF2EXModulus& F, long m)
 
 
 
+NTL_THREAD_LOCAL
 long GF2EXArgBound = 0;
 
 
@@ -3043,6 +3058,9 @@ void TraceMod(GF2E& x, const GF2EX& a, const GF2EXModulus& F)
    if (deg(a) >= n)
       Error("trace: bad args");
 
+   // FIXME: in a thread safe version, we should use
+   // some kind of mutex
+
    if (F.tracevec.length() == 0) 
       ComputeTraceVec(F);
 
@@ -3231,7 +3249,7 @@ void PrepareProjection(vec_vec_GF2& tt, const vec_GF2E& s,
    }
 }
 
-void ProjectedInnerProduct(GF2& x, const vec_GF2E& a, 
+void ProjectedInnerProduct(ref_GF2 x, const vec_GF2E& a, 
                            const vec_vec_GF2& b)
 {
    long n = min(a.length(), b.length());
