@@ -36,7 +36,7 @@ GF2X::GF2X(INIT_SIZE_TYPE, long n)
 
 const GF2X& GF2X::zero()
 {
-   static GF2X z;
+   NTL_THREAD_LOCAL static GF2X z;
    return z;
 }
 
@@ -54,6 +54,65 @@ void GF2X::normalize()
    xrep.QuickSetLength(n);
 }
 
+
+
+void GF2X::SetLength(long n)
+{
+   if (n < 0) {
+      Error("SetLength: negative index");
+      return;
+   }
+
+   long w = (n + NTL_BITS_PER_LONG - 1)/NTL_BITS_PER_LONG;
+   long old_w = xrep.length();
+
+   xrep.SetLength(w);
+
+   long i;
+
+   if (w > old_w) {
+      // zero out new words
+
+      for (i = old_w; i < w; i++)
+         xrep[i] = 0;
+   }
+   else {
+      // zero out high order bits of last word
+
+
+     long wi = n/NTL_BITS_PER_LONG;
+     long bi = n - wi*NTL_BITS_PER_LONG;
+
+     if (bi == 0) return;
+     unsigned long mask = (1UL << bi) - 1UL;
+     xrep[wi] &= mask;
+   }
+}
+
+
+ref_GF2 GF2X::operator[](long i)
+{
+   if (i < 0) Error("GF2X: subscript out of range");
+   long wi = i/NTL_BITS_PER_LONG;
+   if (wi >= xrep.length())  Error("GF2X: subscript out of range");
+   long bi = i - wi*NTL_BITS_PER_LONG;
+   return ref_GF2(INIT_LOOP_HOLE, &xrep[wi], bi);
+}
+
+
+const GF2 GF2X::operator[](long i) const
+{
+   if (i < 0) Error("GF2X: subscript out of range");
+   long wi = i/NTL_BITS_PER_LONG;
+   if (wi >= xrep.length())  Error("GF2X: subscript out of range");
+   long bi = i - wi*NTL_BITS_PER_LONG;
+   return to_GF2((xrep[wi] & (1UL << bi)) != 0);
+}
+
+
+
+
+
 long IsZero(const GF2X& a) 
    { return a.xrep.length() == 0; }
 
@@ -71,7 +130,7 @@ long IsX(const GF2X& a)
    return a.xrep.length() == 1 && a.xrep[0] == 2;
 }
 
-GF2 coeff(const GF2X& a, long i)
+const GF2 coeff(const GF2X& a, long i)
 {
    if (i < 0) return to_GF2(0);
    long wi = i/NTL_BITS_PER_LONG;
@@ -81,7 +140,7 @@ GF2 coeff(const GF2X& a, long i)
    return to_GF2((a.xrep[wi] & (1UL << bi)) != 0);
 }
 
-GF2 LeadCoeff(const GF2X& a)
+const GF2 LeadCoeff(const GF2X& a)
 {
    if (IsZero(a))
       return to_GF2(0);
@@ -89,7 +148,7 @@ GF2 LeadCoeff(const GF2X& a)
       return to_GF2(1);
 }
 
-GF2 ConstTerm(const GF2X& a)
+const GF2 ConstTerm(const GF2X& a)
 {
    if (IsZero(a))
       return to_GF2(0);
@@ -162,7 +221,7 @@ void SetCoeff(GF2X& x, long i, long val)
    long bi = i - wi*NTL_BITS_PER_LONG;
 
    x.xrep[wi] &= ~(1UL << bi);
-   if (wi == n-1) x.normalize();
+   if (wi == n-1 && !x.xrep[wi]) x.normalize();
 }
 
 void SetCoeff(GF2X& x, long i, GF2 a)
@@ -263,7 +322,7 @@ istream & HexInput(istream& s, GF2X& a)
 
 istream & operator>>(istream& s, GF2X& a)   
 {   
-   static ZZ ival;
+   NTL_ZZRegister(ival);
 
    long c;   
    if (!s) Error("bad GF2X input"); 
@@ -976,7 +1035,7 @@ void mul(GF2X& c, const GF2X& a, const GF2X& b)
    // finally: the general case
 
 
-   WordVector mem;
+   NTL_THREAD_LOCAL static WordVector mem;
 
    const _ntl_ulong *ap = a.xrep.elts(), *bp = b.xrep.elts();
    _ntl_ulong *cp;
@@ -1342,9 +1401,9 @@ void mul(GF2X& c, const GF2X& a, const GF2X& b)
    // finally: the general case
 
    
-   static WordVector mem;
-   static WordVector stk;
-   static WordVector vec;
+   NTL_THREAD_LOCAL static WordVector mem;
+   NTL_THREAD_LOCAL static WordVector stk;
+   NTL_THREAD_LOCAL static WordVector vec;
 
    const _ntl_ulong *ap, *bp;
    _ntl_ulong *cp;
@@ -1494,17 +1553,6 @@ void trunc(GF2X& x, const GF2X& a, long m)
 }
       
 
-/****** implementation of vec_GF2X ******/
-
-
-NTL_vector_impl(GF2X,vec_GF2X)
-
-NTL_io_vector_impl(GF2X,vec_GF2X)
-
-NTL_eq_vector_impl(GF2X,vec_GF2X)
-
-
-
 void MulByX(GF2X& x, const GF2X& a)
 {
    long n = a.xrep.length();
@@ -1534,7 +1582,7 @@ void MulByX(GF2X& x, const GF2X& a)
 
 
 
-static _ntl_ulong sqrtab[256] = {
+static const _ntl_ulong sqrtab[256] = {
 
 0UL, 1UL, 4UL, 5UL, 16UL, 17UL, 20UL, 21UL, 64UL, 
 65UL, 68UL, 69UL, 80UL, 81UL, 84UL, 85UL, 256UL, 
@@ -1772,7 +1820,7 @@ void RightShift(GF2X& c, const GF2X& a, long n)
 
 
 
-static _ntl_ulong revtab[256] = {
+static const _ntl_ulong revtab[256] = {
 
 0UL, 128UL, 64UL, 192UL, 32UL, 160UL, 96UL, 224UL, 16UL, 144UL, 
 80UL, 208UL, 48UL, 176UL, 112UL, 240UL, 8UL, 136UL, 72UL, 200UL, 
