@@ -26,7 +26,8 @@ void CheckFinite(quad_float *p)
 static void RowTransform(vec_ZZ& A, vec_ZZ& B, const ZZ& MU1)
 // x = x - y*MU
 {
-   static ZZ T, MU;
+   NTL_ZZRegister(T);
+   NTL_ZZRegister(MU);
    long k;
 
    long n = A.length();
@@ -149,7 +150,8 @@ static void RowTransform(vec_ZZ& A, vec_ZZ& B, const ZZ& MU1,
                          double& max_a, double max_b, long& in_float)
 // x = x - y*MU
 {
-   static ZZ T, MU;
+   NTL_ZZRegister(T);
+   NTL_ZZRegister(MU);
    long k;
    double mu;
 
@@ -299,7 +301,8 @@ static void RowTransform(vec_ZZ& A, vec_ZZ& B, const ZZ& MU1,
 static void RowTransform2(vec_ZZ& A, vec_ZZ& B, const ZZ& MU1)
 // x = x + y*MU
 {
-   static ZZ T, MU;
+   NTL_ZZRegister(T);
+   NTL_ZZRegister(MU);
    long k;
 
    long n = A.length();
@@ -346,6 +349,8 @@ static void RowTransform2(vec_ZZ& A, vec_ZZ& B, const ZZ& MU1)
       }
    }
 }
+
+
 class GivensCache_QP {
 public:
    GivensCache_QP(long m, long n);
@@ -588,14 +593,12 @@ void GivensComputeGS(quad_float **B1, quad_float **mu, quad_float **aux, long k,
       CheckFinite(&p[i]);
 }
 
-static quad_float red_fudge = to_quad_float(0);
-static long log_red = 0;
-
-static long verbose = 0;
-
-static unsigned long NumSwaps = 0;
-static double StartTime = 0;
-static double LastTime = 0;
+NTL_THREAD_LOCAL static quad_float red_fudge = to_quad_float(0);
+NTL_THREAD_LOCAL static long log_red = 0;
+NTL_THREAD_LOCAL static long verbose = 0;
+NTL_THREAD_LOCAL static unsigned long NumSwaps = 0;
+NTL_THREAD_LOCAL static double StartTime = 0;
+NTL_THREAD_LOCAL static double LastTime = 0;
 
 
 
@@ -1020,7 +1023,7 @@ long G_LLL_QP(mat_ZZ& B, mat_ZZ& U, double delta, long deep,
 
 
 
-static vec_quad_float G_BKZConstant;
+NTL_THREAD_LOCAL static vec_quad_float G_BKZConstant;
 
 static
 void ComputeG_BKZConstant(long beta, long p)
@@ -1077,7 +1080,7 @@ void ComputeG_BKZConstant(long beta, long p)
    }
 }
 
-static vec_quad_float G_BKZThresh;
+NTL_THREAD_LOCAL static vec_quad_float G_BKZThresh;
 
 static 
 void ComputeG_BKZThresh(quad_float *c, long beta)
