@@ -12,7 +12,8 @@ NTL_START_IMPL
 static void RowTransform(vec_ZZ& A, vec_ZZ& B, const ZZ& MU1)
 // x = x - y*MU
 {
-   static ZZ T, MU;
+   NTL_ZZRegister(T);
+   NTL_ZZRegister(MU);
    long k;
 
    long n = A.length();
@@ -64,7 +65,8 @@ static void RowTransform(vec_ZZ& A, vec_ZZ& B, const ZZ& MU1)
 static void RowTransform2(vec_ZZ& A, vec_ZZ& B, const ZZ& MU1)
 // x = x + y*MU
 {
-   static ZZ T, MU;
+   NTL_ZZRegister(T);
+   NTL_ZZRegister(MU);
    long k;
 
    long n = A.length();
@@ -389,8 +391,8 @@ void GivensComputeGS(mat_RR& B1, mat_RR& mu, mat_RR& aux, long k, long n,
 
 }
 
-static RR red_fudge;
-static long log_red = 0;
+NTL_THREAD_LOCAL static RR red_fudge;
+NTL_THREAD_LOCAL static long log_red = 0;
 
 static void init_red_fudge()
 {
@@ -414,11 +416,10 @@ static void inc_red_fudge()
 
 
 
-static long verbose = 0;
-
-static unsigned long NumSwaps = 0;
-static double StartTime = 0;
-static double LastTime = 0;
+NTL_THREAD_LOCAL static long verbose = 0;
+NTL_THREAD_LOCAL static unsigned long NumSwaps = 0;
+NTL_THREAD_LOCAL static double StartTime = 0;
+NTL_THREAD_LOCAL static double LastTime = 0;
 
 
 
@@ -755,7 +756,7 @@ long G_LLL_RR(mat_ZZ& B, mat_ZZ& U, double delta, long deep,
 
 
 
-static vec_RR G_BKZConstant;
+NTL_THREAD_LOCAL static vec_RR G_BKZConstant;
 
 static
 void ComputeG_BKZConstant(long beta, long p)
@@ -809,7 +810,7 @@ void ComputeG_BKZConstant(long beta, long p)
 
 }
 
-static vec_RR G_BKZThresh;
+NTL_THREAD_LOCAL static vec_RR G_BKZThresh;
 
 static 
 void ComputeG_BKZThresh(RR *c, long beta)
