@@ -12,7 +12,8 @@ NTL_START_IMPL
 static void RowTransform(vec_ZZ& A, vec_ZZ& B, const ZZ& MU1)
 // x = x - y*MU
 {
-   static ZZ T, MU;
+   NTL_ZZRegister(T);
+   NTL_ZZRegister(MU);
    long k;
 
    long n = A.length();
@@ -76,7 +77,8 @@ static void RowTransform(vec_ZZ& A, vec_ZZ& B, const ZZ& MU1)
 static void RowTransform2(vec_ZZ& A, vec_ZZ& B, const ZZ& MU1)
 // x = x + y*MU
 {
-   static ZZ T, MU;
+   NTL_ZZRegister(T);
+   NTL_ZZRegister(MU);
    long k;
 
    long n = A.length();
@@ -364,8 +366,8 @@ void GivensComputeGS(xdouble **B1, xdouble **mu, xdouble **aux, long k, long n,
    if (k > n) p[k] = 0;
 }
 
-static xdouble red_fudge = to_xdouble(0);
-static long log_red = 0;
+NTL_THREAD_LOCAL static xdouble red_fudge = to_xdouble(0);
+NTL_THREAD_LOCAL static long log_red = 0;
 
 static void init_red_fudge()
 {
@@ -392,11 +394,10 @@ static void inc_red_fudge()
 
 
 
-static long verbose = 0;
-
-static unsigned long NumSwaps = 0;
-static double StartTime = 0;
-static double LastTime = 0;
+NTL_THREAD_LOCAL static long verbose = 0;
+NTL_THREAD_LOCAL static unsigned long NumSwaps = 0;
+NTL_THREAD_LOCAL static double StartTime = 0;
+NTL_THREAD_LOCAL static double LastTime = 0;
 
 
 
@@ -769,7 +770,7 @@ long G_LLL_XD(mat_ZZ& B, mat_ZZ& U, double delta, long deep,
 
 
 
-static vec_xdouble G_BKZConstant;
+NTL_THREAD_LOCAL static vec_xdouble G_BKZConstant;
 
 static
 void ComputeG_BKZConstant(long beta, long p)
@@ -824,7 +825,7 @@ void ComputeG_BKZConstant(long beta, long p)
    }
 }
 
-static vec_xdouble G_BKZThresh;
+NTL_THREAD_LOCAL static vec_xdouble G_BKZThresh;
 
 static
 void ComputeG_BKZThresh(xdouble *c, long beta)
