@@ -34,11 +34,11 @@ static quad_float InnerProduct(quad_float *a, quad_float *b, long n)
    return s;
 }
 
-
 static void RowTransform(vec_ZZ& A, vec_ZZ& B, const ZZ& MU1)
 // x = x - y*MU
 {
-   static ZZ T, MU;
+   NTL_ZZRegister(T);
+   NTL_ZZRegister(MU);
    long k;
 
    long n = A.length();
@@ -72,10 +72,21 @@ static void RowTransform(vec_ZZ& A, vec_ZZ& B, const ZZ& MU1)
       long mu1;
       conv(mu1, MU);
 
-      for (i = 1; i <= n; i++) {
-         mul(T, B(i), mu1);
-         if (k > 0) LeftShift(T, T, k);
-         sub(A(i), A(i), T);
+      if (k > 0) {
+
+         for (i = 1; i <= n; i++) {
+            mul(T, B(i), mu1);
+            LeftShift(T, T, k);
+            sub(A(i), A(i), T);
+         }
+
+      }
+      else {
+
+         for (i = 1; i <= n; i++) {
+            MulSubFrom(A(i), B(i), mu1);
+         }
+
       }
    }
    else {
@@ -87,6 +98,8 @@ static void RowTransform(vec_ZZ& A, vec_ZZ& B, const ZZ& MU1)
    }
 }
 
+
+
 #define TR_BND (NTL_FDOUBLE_PRECISION/2.0)
 // Just to be safe!!
 
@@ -147,7 +160,8 @@ static void RowTransform(vec_ZZ& A, vec_ZZ& B, const ZZ& MU1,
                          double& max_a, double max_b, long& in_float)
 // x = x - y*MU
 {
-   static ZZ T, MU;
+   NTL_ZZRegister(T);
+   NTL_ZZRegister(MU);
    long k;
    double mu;
 
@@ -276,8 +290,7 @@ static void RowTransform(vec_ZZ& A, vec_ZZ& B, const ZZ& MU1,
                   conv(A(i), a[i].hi);
                   in_a[i] = 0;
                }
-               mul(T, B(i), mu1);
-               sub(A(i), A(i), T);
+               MulSubFrom(A(i), B(i), mu1);
            }
          }
       }
@@ -298,7 +311,8 @@ static void RowTransform(vec_ZZ& A, vec_ZZ& B, const ZZ& MU1,
 static void RowTransform2(vec_ZZ& A, vec_ZZ& B, const ZZ& MU1)
 // x = x + y*MU
 {
-   static ZZ T, MU;
+   NTL_ZZRegister(T);
+   NTL_ZZRegister(MU);
    long k;
 
    long n = A.length();
@@ -416,14 +430,12 @@ void ComputeGS(mat_ZZ& B, quad_float **B1, quad_float **mu, quad_float *b,
    c[k] = b[k] - s;
 }
 
-static quad_float red_fudge = to_quad_float(0);
-static long log_red = 0;
-
-static long verbose = 0;
-
-static unsigned long NumSwaps = 0;
-static double StartTime = 0;
-static double LastTime = 0;
+NTL_THREAD_LOCAL static quad_float red_fudge = to_quad_float(0);
+NTL_THREAD_LOCAL static long log_red = 0;
+NTL_THREAD_LOCAL static long verbose = 0;
+NTL_THREAD_LOCAL static unsigned long NumSwaps = 0;
+NTL_THREAD_LOCAL static double StartTime = 0;
+NTL_THREAD_LOCAL static double LastTime = 0;
 
 
 static void LLLStatus(long max_k, double t, long m, const mat_ZZ& B)
@@ -515,7 +527,7 @@ long ll_LLL_QP(mat_ZZ& B, mat_ZZ* U, quad_float delta, long deep,
    quad_float *tp;
 
 
-   static double bound = 0;
+   NTL_THREAD_LOCAL static double bound = 0;
 
 
    if (bound == 0) {
@@ -911,7 +923,7 @@ long LLL_QP(mat_ZZ& B, mat_ZZ& U, double delta, long deep,
 
 
 
-static vec_quad_float BKZConstant;
+NTL_THREAD_LOCAL static vec_quad_float BKZConstant;
 
 static
 void ComputeBKZConstant(long beta, long p)
@@ -968,7 +980,8 @@ void ComputeBKZConstant(long beta, long p)
    }
 }
 
-static vec_quad_float BKZThresh;
+
+NTL_THREAD_LOCAL static vec_quad_float BKZThresh;
 
 static 
 void ComputeBKZThresh(quad_float *c, long beta)
