@@ -13,7 +13,8 @@ NTL_START_IMPL
 static void RowTransform(vec_ZZ& A, vec_ZZ& B, const ZZ& MU1)
 // x = x - y*MU
 {
-   static ZZ T, MU;
+   NTL_ZZRegister(T);
+   NTL_ZZRegister(MU);
    long k;
 
    long n = A.length();
@@ -65,7 +66,8 @@ static void RowTransform(vec_ZZ& A, vec_ZZ& B, const ZZ& MU1)
 static void RowTransform2(vec_ZZ& A, vec_ZZ& B, const ZZ& MU1)
 // x = x + y*MU
 {
-   static ZZ T, MU;
+   NTL_ZZRegister(T);
+   NTL_ZZRegister(MU);
    long k;
 
    long n = A.length();
@@ -160,8 +162,8 @@ void ComputeGS(const mat_ZZ& B, mat_RR& B1,
    sub(c(k), b(k), s);
 }
 
-static RR red_fudge;
-static long log_red = 0;
+NTL_THREAD_LOCAL static RR red_fudge;
+NTL_THREAD_LOCAL static long log_red = 0;
 
 static void init_red_fudge()
 {
@@ -185,11 +187,10 @@ static void inc_red_fudge()
 
 
 
-static long verbose = 0;
-
-static unsigned long NumSwaps = 0;
-static double StartTime = 0;
-static double LastTime = 0;
+NTL_THREAD_LOCAL static long verbose = 0;
+NTL_THREAD_LOCAL static unsigned long NumSwaps = 0;
+NTL_THREAD_LOCAL static double StartTime = 0;
+NTL_THREAD_LOCAL static double LastTime = 0;
 
 
 
@@ -587,7 +588,7 @@ long LLL_RR(mat_ZZ& B, mat_ZZ& U, double delta, long deep,
 
 
 
-static vec_RR BKZConstant;
+NTL_THREAD_LOCAL static vec_RR BKZConstant;
 
 static
 void ComputeBKZConstant(long beta, long p)
@@ -641,7 +642,7 @@ void ComputeBKZConstant(long beta, long p)
 
 }
 
-static vec_RR BKZThresh;
+NTL_THREAD_LOCAL static vec_RR BKZThresh;
 
 static 
 void ComputeBKZThresh(RR *c, long beta)
