@@ -5,11 +5,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 
-#if (defined(__cplusplus) && !defined(NTL_CXX_ONLY))
-extern "C" double _ntl_GetTime();
-#else
-extern double _ntl_GetTime();
-#endif
+double _ntl_GetTime();
 
 /* Assuming the processor speed is at most 200GHz, and that
  * the clock resolution is at least 1 millisecond, the following
@@ -28,11 +24,11 @@ int main(int argc, char **argv)
 
    fprintf(stderr, "running");
 
-   x = atol(argv[1]); // = 1
+   x = atol(argv[1]); /* = 1 */
 
-   n = atol(argv[2]); // = 1048576 = 2^20
+   n = atol(argv[2]); /* = 1048576 = 2^20 */
 
-   m = atol(argv[3]); // = 1048575 = 2^20 - 1
+   m = atol(argv[3]); /* = 1048575 = 2^20 - 1 */
 
    k = -1;
    t0 = _ntl_GetTime();
