@@ -3,6 +3,7 @@
 
 #include <NTL/ZZ_pEX.h>
 #include <NTL/vec_vec_ZZ_p.h>
+#include <NTL/ZZX.h>
 
 #include <NTL/new.h>
 
@@ -11,7 +12,7 @@ NTL_START_IMPL
 
 const ZZ_pEX& ZZ_pEX::zero()
 {
-   static ZZ_pEX z;
+   NTL_THREAD_LOCAL static ZZ_pEX z;
    return z;
 }
 
@@ -309,6 +310,25 @@ void conv(ZZ_pEX& x, const vec_ZZ_pE& a)
 }
 
 
+
+/* additional legacy conversions for v6 conversion regime */
+
+void conv(ZZ_pEX& x, const ZZX& a)
+{
+   long n = a.rep.length();
+   long i;
+
+   x.rep.SetLength(n);
+   for (i = 0; i < n; i++)
+      conv(x.rep[i], a.rep[i]);
+
+   x.normalize();
+}
+
+
+/* ------------------------------------- */
+
+
 void add(ZZ_pEX& x, const ZZ_pEX& a, const ZZ_pEX& b)
 {
    long da = deg(a);
@@ -966,15 +986,15 @@ void PlainMul(ZZ_pEX& x, const ZZ_pEX& a, const ZZ_pEX& b)
    xp = x.rep.elts();
 
    long i, j, jmin, jmax;
-   static ZZ_pX t, accum;
+   ZZ_pX t, accum;
 
    for (i = 0; i <= d; i++) {
       jmin = max(0, i-db);
       jmax = min(da, i);
       clear(accum);
       for (j = jmin; j <= jmax; j++) {
-	 mul(t, rep(ap[j]), rep(bp[i-j]));
-	 add(accum, accum, t);
+         mul(t, rep(ap[j]), rep(bp[i-j]));
+         add(accum, accum, t);
       }
       conv(xp[i], accum);
    }
@@ -1045,13 +1065,13 @@ void PlainDivRem(ZZ_pEX& q, ZZ_pEX& r, const ZZ_pEX& a, const ZZ_pEX& b)
    for (i = dq; i >= 0; i--) {
       conv(t, xp[i+db]);
       if (!LCIsOne)
-	 mul(t, t, LCInv);
+         mul(t, t, LCInv);
       qp[i] = t;
       negate(t, t);
 
       for (j = db-1; j >= 0; j--) {
-	 mul(s, rep(t), rep(bp[j]));
-	 add(xp[i+j], xp[i+j], s);
+         mul(s, rep(t), rep(bp[j]));
+         add(xp[i+j], xp[i+j], s);
       }
    }
 
@@ -1101,12 +1121,12 @@ void PlainRem(ZZ_pEX& r, const ZZ_pEX& a, const ZZ_pEX& b, vec_ZZ_pX& x)
    for (i = dq; i >= 0; i--) {
       conv(t, xp[i+db]);
       if (!LCIsOne)
-	 mul(t, t, LCInv);
+         mul(t, t, LCInv);
       negate(t, t);
 
       for (j = db-1; j >= 0; j--) {
-	 mul(s, rep(t), rep(bp[j]));
-	 add(xp[i+j], xp[i+j], s);
+         mul(s, rep(t), rep(bp[j]));
+         add(xp[i+j], xp[i+j], s);
       }
    }
 
@@ -1168,13 +1188,13 @@ void PlainDivRem(ZZ_pEX& q, ZZ_pEX& r, const ZZ_pEX& a, const ZZ_pEX& b,
    for (i = dq; i >= 0; i--) {
       conv(t, xp[i+db]);
       if (!LCIsOne)
-	 mul(t, t, LCInv);
+         mul(t, t, LCInv);
       qp[i] = t;
       negate(t, t);
 
       for (j = db-1; j >= 0; j--) {
-	 mul(s, rep(t), rep(bp[j]));
-	 add(xp[i+j], xp[i+j], s);
+         mul(s, rep(t), rep(bp[j]));
+         add(xp[i+j], xp[i+j], s);
       }
    }
 
@@ -1237,15 +1257,15 @@ void PlainDiv(ZZ_pEX& q, const ZZ_pEX& a, const ZZ_pEX& b)
    for (i = dq; i >= 0; i--) {
       conv(t, xp[i]);
       if (!LCIsOne)
-	 mul(t, t, LCInv);
+         mul(t, t, LCInv);
       qp[i] = t;
       negate(t, t);
 
       long lastj = max(0, db-i);
 
       for (j = db-1; j >= lastj; j--) {
-	 mul(s, rep(t), rep(bp[j]));
-	 add(xp[i+j-db], xp[i+j-db], s);
+         mul(s, rep(t), rep(bp[j]));
+         add(xp[i+j-db], xp[i+j-db], s);
       }
    }
 }
@@ -1292,12 +1312,12 @@ void PlainRem(ZZ_pEX& r, const ZZ_pEX& a, const ZZ_pEX& b)
    for (i = dq; i >= 0; i--) {
       conv(t, xp[i+db]);
       if (!LCIsOne)
-	 mul(t, t, LCInv);
+         mul(t, t, LCInv);
       negate(t, t);
 
       for (j = db-1; j >= 0; j--) {
-	 mul(s, rep(t), rep(bp[j]));
-	 add(xp[i+j], xp[i+j], s);
+         mul(s, rep(t), rep(bp[j]));
+         add(xp[i+j], xp[i+j], s);
       }
    }
 
@@ -1385,7 +1405,7 @@ void NewtonInv(ZZ_pEX& c, const ZZ_pEX& a, long e)
       return;
    }
 
-   static vec_long E;
+   vec_long E;
    E.SetLength(0);
    append(E, e);
    while (e > 1) {
@@ -1965,12 +1985,6 @@ void XGCD(ZZ_pEX& d, ZZ_pEX& s, ZZ_pEX& t, const ZZ_pEX& a, const ZZ_pEX& b)
    mul(t, t, z);
 }
 
-NTL_vector_impl(ZZ_pEX,vec_ZZ_pEX)
-
-NTL_eq_vector_impl(ZZ_pEX,vec_ZZ_pEX)
-
-NTL_io_vector_impl(ZZ_pEX,vec_ZZ_pEX)
-
 void IterBuild(ZZ_pE* a, long n)
 {
    long i, k;
@@ -2195,7 +2209,7 @@ void build(ZZ_pEXArgument& A, const ZZ_pEX& h, const ZZ_pEXModulus& F, long m)
       MulMod(A.H[i], A.H[i-1], h, F);
 }
 
-long ZZ_pEXArgBound = 0;
+NTL_THREAD_LOCAL long ZZ_pEXArgBound = 0;
 
 
 
@@ -3020,6 +3034,8 @@ void TraceMod(ZZ_pE& x, const ZZ_pEX& a, const ZZ_pEXModulus& F)
    if (deg(a) >= n)
       Error("trace: bad args");
 
+  // FIXME: in a thread-safe version, this needs to be mutexed
+
    if (F.tracevec.length() == 0) 
       ComputeTraceVec(F);
 
@@ -3081,9 +3097,8 @@ void PlainResultant(ZZ_pE& rres, const ZZ_pEX& a, const ZZ_pEX& b)
             break;
          }
       }
-
-      rres = res;
    }
+   rres = res;
 }
 
 void resultant(ZZ_pE& rres, const ZZ_pEX& a, const ZZ_pEX& b)
