@@ -175,7 +175,8 @@ void mul(ZZ_pX& U, ZZ_pX& V, const ZZ_pXMatrix& M)
    long n = (1L << k);
    long xx;
    ZZ_p a0, a1, b0, b1, c0, d0, u0, u1, v0, v1, nu0, nu1, nv0;
-   static ZZ t1, t2;
+   NTL_ZZRegister(t1);
+   NTL_ZZRegister(t2);
 
    if (n == d-1)
       xx = 1;
@@ -667,7 +668,8 @@ void IterBuild(ZZ_p* a, long n)
 
 void mul(ZZ_p* x, const ZZ_p* a, const ZZ_p* b, long n)
 {
-   static ZZ t, accum;
+   NTL_ZZRegister(t);
+   NTL_ZZRegister(accum);
 
    long i, j, jmin, jmax;
 
@@ -895,19 +897,12 @@ void interpolate(ZZ_pX& f, const vec_ZZ_p& a, const vec_ZZ_p& b)
    f.rep = res;
 }
 
-NTL_vector_impl(ZZ_pX,vec_ZZ_pX)
-
-NTL_eq_vector_impl(ZZ_pX,vec_ZZ_pX)
-
-NTL_io_vector_impl(ZZ_pX,vec_ZZ_pX)
-
-
 
    
 void InnerProduct(ZZ_pX& x, const vec_ZZ_p& v, long low, long high, 
                    const vec_ZZ_pX& H, long n, ZZVec& t)
 {
-   static ZZ s;
+   NTL_ZZRegister(s);
    long i, j;
 
    for (j = 0; j < n; j++)
@@ -993,7 +988,7 @@ void build(ZZ_pXArgument& A, const ZZ_pX& h, const ZZ_pXModulus& F, long m)
 
 
 
-long ZZ_pXArgBound = 0;
+NTL_THREAD_LOCAL long ZZ_pXArgBound = 0;
 
 
 void CompMod(ZZ_pX& x, const ZZ_pX& g, const ZZ_pX& h, const ZZ_pXModulus& F)
@@ -1622,6 +1617,9 @@ void TraceMod(ZZ_p& x, const ZZ_pX& a, const ZZ_pXModulus& F)
    if (deg(a) >= n)
       Error("trace: bad args");
 
+   // FIXME: in a thread safe version, we should use
+   // some kind of mutex
+
    if (F.tracevec.length() == 0) 
       ComputeTraceVec(F);
 
@@ -1681,9 +1679,9 @@ void PlainResultant(ZZ_p& rres, const ZZ_pX& a, const ZZ_pX& b)
             break;
          }
       }
-
-      rres = res;
    }
+
+   rres = res;
 }
 
 
