@@ -4,6 +4,7 @@
 
 #include <NTL/lzz_pEX.h>
 #include <NTL/vec_vec_lzz_p.h>
+#include <NTL/ZZX.h>
 
 #include <NTL/new.h>
 
@@ -12,7 +13,7 @@ NTL_START_IMPL
 
 const zz_pEX& zz_pEX::zero()
 {
-   static zz_pEX z;
+   NTL_THREAD_LOCAL static zz_pEX z;
    return z;
 }
 
@@ -311,6 +312,27 @@ void conv(zz_pEX& x, const vec_zz_pE& a)
 }
 
 
+
+
+/* additional legacy conversions for v6 conversion regime */
+
+void conv(zz_pEX& x, const ZZX& a)
+{
+   long n = a.rep.length();
+   long i;
+
+   x.rep.SetLength(n);
+   for (i = 0; i < n; i++)
+      conv(x.rep[i], a.rep[i]);
+
+   x.normalize();
+}
+
+
+/* ------------------------------------- */
+
+
+
 void add(zz_pEX& x, const zz_pEX& a, const zz_pEX& b)
 {
    long da = deg(a);
@@ -969,15 +991,15 @@ void PlainMul(zz_pEX& x, const zz_pEX& a, const zz_pEX& b)
    xp = x.rep.elts();
 
    long i, j, jmin, jmax;
-   static zz_pX t, accum;
+   zz_pX t, accum;
 
    for (i = 0; i <= d; i++) {
       jmin = max(0, i-db);
       jmax = min(da, i);
       clear(accum);
       for (j = jmin; j <= jmax; j++) {
-	 mul(t, rep(ap[j]), rep(bp[i-j]));
-	 add(accum, accum, t);
+         mul(t, rep(ap[j]), rep(bp[i-j]));
+         add(accum, accum, t);
       }
       conv(xp[i], accum);
    }
@@ -1048,13 +1070,13 @@ void PlainDivRem(zz_pEX& q, zz_pEX& r, const zz_pEX& a, const zz_pEX& b)
    for (i = dq; i >= 0; i--) {
       conv(t, xp[i+db]);
       if (!LCIsOne)
-	 mul(t, t, LCInv);
+         mul(t, t, LCInv);
       qp[i] = t;
       negate(t, t);
 
       for (j = db-1; j >= 0; j--) {
-	 mul(s, rep(t), rep(bp[j]));
-	 add(xp[i+j], xp[i+j], s);
+         mul(s, rep(t), rep(bp[j]));
+         add(xp[i+j], xp[i+j], s);
       }
    }
 
@@ -1104,12 +1126,12 @@ void PlainRem(zz_pEX& r, const zz_pEX& a, const zz_pEX& b, vec_zz_pX& x)
    for (i = dq; i >= 0; i--) {
       conv(t, xp[i+db]);
       if (!LCIsOne)
-	 mul(t, t, LCInv);
+         mul(t, t, LCInv);
       negate(t, t);
 
       for (j = db-1; j >= 0; j--) {
-	 mul(s, rep(t), rep(bp[j]));
-	 add(xp[i+j], xp[i+j], s);
+         mul(s, rep(t), rep(bp[j]));
+         add(xp[i+j], xp[i+j], s);
       }
    }
 
@@ -1171,13 +1193,13 @@ void PlainDivRem(zz_pEX& q, zz_pEX& r, const zz_pEX& a, const zz_pEX& b,
    for (i = dq; i >= 0; i--) {
       conv(t, xp[i+db]);
       if (!LCIsOne)
-	 mul(t, t, LCInv);
+         mul(t, t, LCInv);
       qp[i] = t;
       negate(t, t);
 
       for (j = db-1; j >= 0; j--) {
-	 mul(s, rep(t), rep(bp[j]));
-	 add(xp[i+j], xp[i+j], s);
+         mul(s, rep(t), rep(bp[j]));
+         add(xp[i+j], xp[i+j], s);
       }
    }
 
@@ -1240,15 +1262,15 @@ void PlainDiv(zz_pEX& q, const zz_pEX& a, const zz_pEX& b)
    for (i = dq; i >= 0; i--) {
       conv(t, xp[i]);
       if (!LCIsOne)
-	 mul(t, t, LCInv);
+         mul(t, t, LCInv);
       qp[i] = t;
       negate(t, t);
 
       long lastj = max(0, db-i);
 
       for (j = db-1; j >= lastj; j--) {
-	 mul(s, rep(t), rep(bp[j]));
-	 add(xp[i+j-db], xp[i+j-db], s);
+         mul(s, rep(t), rep(bp[j]));
+         add(xp[i+j-db], xp[i+j-db], s);
       }
    }
 }
@@ -1295,12 +1317,12 @@ void PlainRem(zz_pEX& r, const zz_pEX& a, const zz_pEX& b)
    for (i = dq; i >= 0; i--) {
       conv(t, xp[i+db]);
       if (!LCIsOne)
-	 mul(t, t, LCInv);
+         mul(t, t, LCInv);
       negate(t, t);
 
       for (j = db-1; j >= 0; j--) {
-	 mul(s, rep(t), rep(bp[j]));
-	 add(xp[i+j], xp[i+j], s);
+         mul(s, rep(t), rep(bp[j]));
+         add(xp[i+j], xp[i+j], s);
       }
    }
 
@@ -1388,7 +1410,7 @@ void NewtonInv(zz_pEX& c, const zz_pEX& a, long e)
       return;
    }
 
-   static vec_long E;
+   vec_long E;
    E.SetLength(0);
    append(E, e);
    while (e > 1) {
@@ -1967,11 +1989,6 @@ void XGCD(zz_pEX& d, zz_pEX& s, zz_pEX& t, const zz_pEX& a, const zz_pEX& b)
    mul(t, t, z);
 }
 
-NTL_vector_impl(zz_pEX,vec_zz_pEX)
-
-NTL_eq_vector_impl(zz_pEX,vec_zz_pEX)
-
-NTL_io_vector_impl(zz_pEX,vec_zz_pEX)
 
 void IterBuild(zz_pE* a, long n)
 {
@@ -2198,7 +2215,7 @@ void build(zz_pEXArgument& A, const zz_pEX& h, const zz_pEXModulus& F, long m)
       MulMod(A.H[i], A.H[i-1], h, F);
 }
 
-long zz_pEXArgBound = 0;
+NTL_THREAD_LOCAL long zz_pEXArgBound = 0;
 
 
 
@@ -3023,6 +3040,8 @@ void TraceMod(zz_pE& x, const zz_pEX& a, const zz_pEXModulus& F)
    if (deg(a) >= n)
       Error("trace: bad args");
 
+   // FIXME: in a thread-safe impl, we need some kind of mutex here
+
    if (F.tracevec.length() == 0) 
       ComputeTraceVec(F);
 
@@ -3084,9 +3103,8 @@ void PlainResultant(zz_pE& rres, const zz_pEX& a, const zz_pEX& b)
             break;
          }
       }
-
-      rres = res;
    }
+   rres = res;
 }
 
 void resultant(zz_pE& rres, const zz_pEX& a, const zz_pEX& b)
