@@ -7,23 +7,23 @@
 NTL_START_IMPL
 
 
+// NOTE: these are declared extern in lzz_pX.h
 
-long zz_pX_mod_crossover[5] = {45, 45, 90, 180, 180};
-long zz_pX_mul_crossover[5] = {90, 400, 600, 1500, 1500};
-long zz_pX_newton_crossover[5] = {150, 150, 300, 700, 700};
-long zz_pX_div_crossover[5] = {180, 180, 350, 750, 750};
-long zz_pX_halfgcd_crossover[5] = {90, 90, 180, 350, 350};
-long zz_pX_gcd_crossover[5] = {400, 400, 800, 1400, 1400};
-long zz_pX_bermass_crossover[5] = {400, 480, 900, 1600, 1600};
-long zz_pX_trace_crossover[5] = {200, 350, 450, 800, 800};
+const long zz_pX_mod_crossover[5] = {45, 45, 90, 180, 180};
+const long zz_pX_mul_crossover[5] = {90, 400, 600, 1500, 1500};
+const long zz_pX_newton_crossover[5] = {150, 150, 300, 700, 700};
+const long zz_pX_div_crossover[5] = {180, 180, 350, 750, 750};
+const long zz_pX_halfgcd_crossover[5] = {90, 90, 180, 350, 350};
+const long zz_pX_gcd_crossover[5] = {400, 400, 800, 1400, 1400};
+const long zz_pX_bermass_crossover[5] = {400, 480, 900, 1600, 1600};
+const long zz_pX_trace_crossover[5] = {200, 350, 450, 800, 800};
 
-#define QUICK_CRT (NTL_DOUBLE_PRECISION - NTL_SP_NBITS > 12)
 
 
 
 const zz_pX& zz_pX::zero()
 {
-   static zz_pX z;
+   NTL_THREAD_LOCAL static zz_pX z;
    return z;
 }
 
@@ -143,7 +143,7 @@ long IsX(const zz_pX& a)
       
       
 
-zz_p coeff(const zz_pX& a, long i)
+const zz_p coeff(const zz_pX& a, long i)
 {
    if (i < 0 || i > deg(a))
       return zz_p::zero();
@@ -152,7 +152,7 @@ zz_p coeff(const zz_pX& a, long i)
 }
 
 
-zz_p LeadCoeff(const zz_pX& a)
+const zz_p LeadCoeff(const zz_pX& a)
 {
    if (IsZero(a))
       return zz_p::zero();
@@ -160,7 +160,7 @@ zz_p LeadCoeff(const zz_pX& a)
       return a.rep[deg(a)];
 }
 
-zz_p ConstTerm(const zz_pX& a)
+const zz_p ConstTerm(const zz_pX& a)
 {
    if (IsZero(a))
       return zz_p::zero();
@@ -367,7 +367,7 @@ void PlainMul(zz_p *xp, const zz_p *ap, long sa, const zz_p *bp, long sb)
    }
 }
 
-static vec_double a_buf, b_buf;
+NTL_THREAD_LOCAL static vec_double a_buf, b_buf;
 
 static inline 
 void reduce(zz_p& r, double x, long p, double pinv)
@@ -958,7 +958,7 @@ void PlainDivRem(zz_pX& q, zz_pX& r, const zz_pX& a, const zz_pX& b)
    for (i = dq; i >= 0; i--) {
       t = xp[i+db];
       if (!LCIsOne)
-	 mul(t, t, LCInv);
+         mul(t, t, LCInv);
       qp[i] = t;
       negate(t, t);
 
@@ -1036,7 +1036,7 @@ void PlainDiv(zz_pX& q, const zz_pX& a, const zz_pX& b)
    for (i = dq; i >= 0; i--) {
       t = xp[i];
       if (!LCIsOne)
-	 mul(t, t, LCInv);
+         mul(t, t, LCInv);
       qp[i] = t;
       negate(t, t);
 
@@ -1100,7 +1100,7 @@ void PlainRem(zz_pX& r, const zz_pX& a, const zz_pX& b)
    for (i = dq; i >= 0; i--) {
       t = xp[i+db];
       if (!LCIsOne)
-	 mul(t, t, LCInv);
+         mul(t, t, LCInv);
       negate(t, t);
 
       long T = rep(t);
@@ -1383,7 +1383,9 @@ void fftRep::SetSize(long NewK)
       return;
    }
 
-   if (NumPrimes != zz_pInfo->NumPrimes)
+   if (MaxK == -1) 
+      NumPrimes = zz_pInfo->NumPrimes;
+   else if (NumPrimes != zz_pInfo->NumPrimes)
       Error("fftRep: inconsistent use");
 
    long i, n;
@@ -1405,10 +1407,12 @@ void fftRep::SetSize(long NewK)
 fftRep::fftRep(const fftRep& R)
 {
    k = MaxK = R.k;
-   NumPrimes = R.NumPrimes;
+   NumPrimes = 0;
 
    if (k < 0) return;
 
+   NumPrimes = R.NumPrimes;
+
    long i, j, n;
 
    n = 1L << k;
@@ -1426,7 +1430,7 @@ fftRep& fftRep::operator=(const fftRep& R)
 {
    if (this == &R) return *this;
 
-   if (NumPrimes != R.NumPrimes)
+   if (MaxK >= 0 && R.MaxK >= 0 && NumPrimes != R.NumPrimes)
       Error("fftRep: inconsistent use");
 
    if (R.k < 0) {
@@ -1434,6 +1438,8 @@ fftRep& fftRep::operator=(const fftRep& R)
       return *this;
    }
 
+   NumPrimes = R.NumPrimes;
+
    if (R.k > MaxK) {
       long i, n;
 
@@ -1479,29 +1485,31 @@ fftRep::~fftRep()
 
 
 
-static vec_long FFTBuf;
 
 
-
-
-void FromModularRep(zz_p& x, long *a)
+void FromModularRep(zz_p& res, long *a)
 {
    long n = zz_pInfo->NumPrimes;
    long p = zz_pInfo->p;
    double pinv = zz_pInfo->pinv;
+   long *CoeffModP = zz_pInfo->CoeffModP;
+   double *x = zz_pInfo->x;
+   long *u = zz_pInfo->u;
+   long MinusMModP = zz_pInfo->MinusMModP;
+
    long q, s, t;
    long i;
    double y;
 
 
+
 // I've re-written the following code in v5.3 so that it is 
 // a bit more robust.  
 
-#if QUICK_CRT
-
+#if (NTL_zz_p_QUICK_CRT)
    y = 0;
    for (i = 0; i < n; i++)
-      y = y + ((double) a[i])*zz_pInfo->x[i];
+      y = y + ((double) a[i])*x[i];
 
    y = floor(y + 0.5);
    y = y - floor(y*pinv)*double(p);
@@ -1509,37 +1517,47 @@ void FromModularRep(zz_p& x, long *a)
    while (y < 0) y += p;
    q = long(y);
 
-#else
-
-   long Q, r;
-
-   y = 0;
-   q = 0;
-
-   for (i = 0; i < n; i++) {
-      r = MulDivRem(Q, a[i], zz_pInfo->u[i], FFTPrime[i], zz_pInfo->x[i]);
-      q = q + Q;
-#if (NTL_BITS_PER_LONG - NTL_SP_NBITS <= 4)
-      // on typical platforms, this reduction will not be necessary.
-      q = q % p;
-#endif
-      y = y + r*FFTPrimeInv[i];
-   }
-
-   q = (q + long(y + 0.5)) % p;
-
-#endif
-
    t = 0;
    for (i = 0; i < n; i++) {
-      s = MulMod(a[i], zz_pInfo->CoeffModP[i], p, pinv);
+
+      // DIRT: uses undocumented MulMod feature (see ZZ.h)
+      // a[i] is not reduced mod p
+      s = MulMod(a[i], CoeffModP[i], p, pinv);
+
       t = AddMod(t, s, p);
    }
       
 
-   s = MulMod(q, zz_pInfo->MinusMModP, p, pinv);
+   s = MulMod(q, MinusMModP, p, pinv);
    t = AddMod(t, s, p);
-   x.LoopHole() = t;
+   res.LoopHole() = t;
+#else
+
+   y = 0;
+   t = 0;
+
+   for (i = 0; i < n; i++) {
+      s = MulMod2(a[i], u[i], FFTPrime[i], x[i]);
+      y = y + double(s)*FFTPrimeInv[i];
+
+
+      // DIRT: uses undocumented MulMod feature (see ZZ.h)
+      // a[i] is not reduced mod p
+      s = MulMod(s, CoeffModP[i], p, pinv);
+
+      t = AddMod(t, s, p);
+   }
+
+   q = (long) (y + 0.5);
+
+   // DIRT: uses undocumented MulMod feature (see ZZ.h)
+   // q may not be reduced mod p
+   s = MulMod(q, MinusMModP, p, pinv);
+
+   t = AddMod(t, s, p);
+   res.LoopHole() = t;
+#endif
+
 }
 
 
@@ -1549,7 +1567,6 @@ void TofftRep(fftRep& y, const zz_pX& x, long k, long lo, long hi)
 // if deg(x) >= 2^k, then x is first reduced modulo X^n-1.
 {
    long n, i, j, m, j1;
-   vec_long& s = FFTBuf;;
    zz_p accum;
    long NumPrimes = zz_pInfo->NumPrimes;
 
@@ -1570,9 +1587,9 @@ void TofftRep(fftRep& y, const zz_pX& x, long k, long lo, long hi)
 
    const zz_p *xx = x.rep.elts();
 
-   long index = zz_pInfo->index;
+   FFTPrimeInfo *p_info = zz_pInfo->p_info;
 
-   if (index >= 0) {
+   if (p_info) {
       for (j = 0; j < n; j++) {
          if (j >= m) {
             y.tbl[0][j] = 0;
@@ -1606,23 +1623,14 @@ void TofftRep(fftRep& y, const zz_pX& x, long k, long lo, long hi)
    }
    
 
-   s.SetLength(n);
-   long *sp = s.elts();
-
-   if (index >= 0) {
-      long *Root = &RootTable[index][0];
+   if (p_info) {
       long *yp = &y.tbl[0][0];
-      FFT(sp, yp, y.k, FFTPrime[index], Root);
-      for (j = 0; j < n; j++)
-         yp[j] = sp[j];
+      FFTFwd(yp, yp, k, *p_info);
    } 
    else {
       for (i = 0; i < zz_pInfo->NumPrimes; i++) {
-         long *Root = &RootTable[i][0];
          long *yp = &y.tbl[i][0];
-         FFT(sp, yp, y.k, FFTPrime[i], Root);
-         for (j = 0; j < n; j++)
-            yp[j] = sp[j];
+         FFTFwd(yp, yp, k, i);
       }
    }
 }
@@ -1636,7 +1644,6 @@ void RevTofftRep(fftRep& y, const vec_zz_p& x,
 
 {
    long n, i, j, m, j1;
-   vec_long& s = FFTBuf;
    zz_p accum;
    long NumPrimes = zz_pInfo->NumPrimes;
 
@@ -1656,11 +1663,11 @@ void RevTofftRep(fftRep& y, const vec_zz_p& x,
 
    const zz_p *xx = x.elts();
 
-   long index = zz_pInfo->index;
+   FFTPrimeInfo *p_info = zz_pInfo->p_info;
 
    offset = offset & (n-1);
 
-   if (index >= 0) {
+   if (p_info) {
       for (j = 0; j < n; j++) {
          if (j >= m) {
             y.tbl[0][offset] = 0;
@@ -1696,29 +1703,14 @@ void RevTofftRep(fftRep& y, const vec_zz_p& x,
    }
 
 
-   s.SetLength(n);
-   long *sp = s.elts();
-
-   if (index >= 0) {
-      long *Root = &RootInvTable[index][0];
+   if (p_info) {
       long *yp = &y.tbl[0][0];
-      long w = TwoInvTable[index][k];
-      long q = FFTPrime[index];
-      double qinv = ((double) 1)/((double) q);
-      FFT(sp, yp, y.k, q, Root);
-      for (j = 0; j < n; j++)
-         yp[j] = MulMod(sp[j], w, q, qinv);
+      FFTRev1(yp, yp, k, *p_info);
    }
    else {
       for (i = 0; i < zz_pInfo->NumPrimes; i++) {
-         long *Root = &RootInvTable[i][0];
          long *yp = &y.tbl[i][0];
-         long w = TwoInvTable[i][k];
-         long q = FFTPrime[i];
-         double qinv = ((double) 1)/((double) q);
-         FFT(sp, yp, y.k, q, Root);
-         for (j = 0; j < n; j++)
-            yp[j] = MulMod(sp[j], w, q, qinv);
+         FFTRev1(yp, yp, k, i);
       }
    }
 }
@@ -1734,38 +1726,20 @@ void FromfftRep(zz_pX& x, fftRep& y, long lo, long hi)
    long NumPrimes = zz_pInfo->NumPrimes;
 
    long t[4];
-   vec_long& s = FFTBuf;
 
    k = y.k;
    n = (1L << k);
 
-   s.SetLength(n);
-   long *sp = s.elts();
+   FFTPrimeInfo *p_info = zz_pInfo->p_info;
 
-   long index = zz_pInfo->index;
-
-   if (index >= 0) {
+   if (p_info) {
       long *yp = &y.tbl[0][0];
-      long q = FFTPrime[index];
-      double qinv = FFTPrimeInv[index];
-      long w = TwoInvTable[index][k];
-      long *Root = &RootInvTable[index][0];
-
-      FFT(sp, yp, k, q, Root);
-
-      for (j = 0; j < n; j++) yp[j] = MulMod(sp[j], w, q, qinv);
+      FFTRev1(yp, yp, k, *p_info);
    }
    else {
       for (i = 0; i < NumPrimes; i++) {
          long *yp = &y.tbl[i][0];
-         long q = FFTPrime[i];
-         double qinv = FFTPrimeInv[i];
-         long w = TwoInvTable[i][k];
-         long *Root = &RootInvTable[i][0];
-   
-         FFT(sp, yp, k, q, Root);
-   
-         for (j = 0; j < n; j++) yp[j] = MulMod(sp[j], w, q, qinv);
+         FFTRev1(yp, yp, k, i);
       }
    }
 
@@ -1774,7 +1748,7 @@ void FromfftRep(zz_pX& x, fftRep& y, long lo, long hi)
    l = max(l, 0);
    x.rep.SetLength(l);
 
-   if (index >= 0) {
+   if (p_info) {
       zz_p *xp = x.rep.elts();
       long *yp = &y.tbl[0][0];
       for (j = 0; j < l; j++) 
@@ -1804,34 +1778,20 @@ void RevFromfftRep(vec_zz_p& x, fftRep& y, long lo, long hi)
    long NumPrimes = zz_pInfo->NumPrimes;
 
    long t[4];
-   vec_long& s = FFTBuf;
 
    k = y.k;
    n = (1L << k);
 
-   s.SetLength(n);
-   long *sp = s.elts();
+   FFTPrimeInfo *p_info = zz_pInfo->p_info;
 
-   long index = zz_pInfo->index;
-
-   if (index >= 0) {
+   if (p_info) {
       long *yp = &y.tbl[0][0];
-      long q = FFTPrime[index];
-      long *Root = &RootTable[index][0];
-
-      FFT(sp, yp, k, q, Root);
-      for (j = 0; j < n; j++)
-         yp[j] = sp[j];
+      FFTFwd(yp, yp, k, *p_info);
    }
    else {
       for (i = 0; i < NumPrimes; i++) {
          long *yp = &y.tbl[i][0];
-         long q = FFTPrime[i];
-         long *Root = &RootTable[i][0];
-   
-         FFT(sp, yp, k, q, Root);
-         for (j = 0; j < n; j++)
-            yp[j] = sp[j];
+         FFTFwd(yp, yp, k, i);
       }
    }
 
@@ -1840,7 +1800,7 @@ void RevFromfftRep(vec_zz_p& x, fftRep& y, long lo, long hi)
    l = max(l, 0);
    x.SetLength(l);
 
-   if (index >= 0) {
+   if (p_info) {
       zz_p *xp = x.elts();
       long *yp = &y.tbl[0][0];
       for (j = 0; j < l; j++) 
@@ -1868,30 +1828,18 @@ void NDFromfftRep(zz_pX& x, const fftRep& y, long lo, long hi, fftRep& z)
 
    z.SetSize(k);
 
-   long index = zz_pInfo->index;
+   FFTPrimeInfo *p_info = zz_pInfo->p_info;
 
-   if (index >= 0) {
+   if (p_info) {
       long *zp = &z.tbl[0][0];
-      long q = FFTPrime[index];
-      double qinv = FFTPrimeInv[index];
-      long w = TwoInvTable[index][k];
-      long *Root = &RootInvTable[index][0];
-
-      FFT(zp, &y.tbl[0][0], k, q, Root);
-
-      for (j = 0; j < n; j++) zp[j] = MulMod(zp[j], w, q, qinv);
+      const long *yp = &y.tbl[0][0];
+      FFTRev1(zp, yp, k, *p_info);
    }
    else {
       for (i = 0; i < NumPrimes; i++) {
          long *zp = &z.tbl[i][0];
-         long q = FFTPrime[i];
-         double qinv = FFTPrimeInv[i];
-         long w = TwoInvTable[i][k];
-         long *Root = &RootInvTable[i][0];
-   
-         FFT(zp, &y.tbl[i][0], k, q, Root);
-   
-         for (j = 0; j < n; j++) zp[j] = MulMod(zp[j], w, q, qinv);
+         const long *yp = &y.tbl[i][0];
+         FFTRev1(zp, yp, k, i);
       }
    }
 
@@ -1900,7 +1848,7 @@ void NDFromfftRep(zz_pX& x, const fftRep& y, long lo, long hi, fftRep& z)
    l = max(l, 0);
    x.rep.SetLength(l);
 
-   if (index >= 0) {
+   if (p_info) {
       zz_p *xp = x.rep.elts();
       long *zp = &z.tbl[0][0];
       for (j = 0; j < l; j++) 
@@ -1935,25 +1883,15 @@ void FromfftRep(zz_p* x, fftRep& y, long lo, long hi)
    long NumPrimes = zz_pInfo->NumPrimes;
 
    long t[4];
-   vec_long& s = FFTBuf;
 
    k = y.k;
    n = (1L << k);
 
-   s.SetLength(n);
-   long *sp = s.elts();
+   FFTPrimeInfo *p_info = zz_pInfo->p_info;
 
-   long index = zz_pInfo->index;
-   if (index >= 0) {
+   if (p_info) {
       long *yp = &y.tbl[0][0];
-      long q = FFTPrime[index];
-      double qinv = FFTPrimeInv[index];
-      long w = TwoInvTable[index][k];
-      long *Root = &RootInvTable[index][0];
-
-      FFT(sp, yp, k, q, Root);
-
-      for (j = 0; j < n; j++) yp[j] = MulMod(sp[j], w, q, qinv);
+      FFTRev1(yp, yp, k, *p_info);
 
       for (j = lo; j <= hi; j++) {
          if (j >= n)
@@ -1966,14 +1904,7 @@ void FromfftRep(zz_p* x, fftRep& y, long lo, long hi)
    else {
       for (i = 0; i < NumPrimes; i++) {
          long *yp = &y.tbl[i][0];
-         long q = FFTPrime[i];
-         double qinv = FFTPrimeInv[i];
-         long w = TwoInvTable[i][k];
-         long *Root = &RootInvTable[i][0];
-   
-         FFT(sp, yp, k, q, Root);
-   
-         for (j = 0; j < n; j++) yp[j] = MulMod(sp[j], w, q, qinv);
+         FFTRev1(yp, yp, k, i);
       }
    
       for (j = lo; j <= hi; j++) {
@@ -2001,14 +1932,14 @@ void mul(fftRep& z, const fftRep& x, const fftRep& y)
 
    z.SetSize(k);
 
-   long index = zz_pInfo->index;
+   FFTPrimeInfo *p_info = zz_pInfo->p_info;
 
-   if (index >= 0) {
+   if (p_info) {
       long *zp = &z.tbl[0][0];
       const long *xp = &x.tbl[0][0];
       const long *yp = &y.tbl[0][0];
-      long q = FFTPrime[index];
-      double qinv = FFTPrimeInv[index];
+      long q = p_info->q;
+      double qinv = p_info->qinv;
 
       for (j = 0; j < n; j++)
          zp[j] = MulMod(xp[j], yp[j], q, qinv);
@@ -2038,13 +1969,13 @@ void sub(fftRep& z, const fftRep& x, const fftRep& y)
 
    z.SetSize(k);
 
-   long index = zz_pInfo->index;
+   FFTPrimeInfo *p_info = zz_pInfo->p_info;
 
-   if (index >= 0) {
+   if (p_info) {
       long *zp = &z.tbl[0][0];
       const long *xp = &x.tbl[0][0];
       const long *yp = &y.tbl[0][0];
-      long q = FFTPrime[index];
+      long q = p_info->q;
 
       for (j = 0; j < n; j++)
          zp[j] = SubMod(xp[j], yp[j], q);
@@ -2073,13 +2004,13 @@ void add(fftRep& z, const fftRep& x, const fftRep& y)
 
    z.SetSize(k);
 
-   long index = zz_pInfo->index;
+   FFTPrimeInfo *p_info = zz_pInfo->p_info;
 
-   if (index >= 0) {
+   if (p_info) {
       long *zp = &z.tbl[0][0];
       const long *xp = &x.tbl[0][0];
       const long *yp = &y.tbl[0][0];
-      long q = FFTPrime[index];
+      long q = p_info->q;
 
       for (j = 0; j < n; j++)
          zp[j] = AddMod(xp[j], yp[j], q);
@@ -2132,10 +2063,10 @@ void AddExpand(fftRep& x, const fftRep& a)
 
    if (l < k) Error("AddExpand: bad args");
 
-   long index = zz_pInfo->index;
+   FFTPrimeInfo *p_info = zz_pInfo->p_info;
    
-   if (index >= 0) {
-      long q = FFTPrime[index];
+   if (p_info) {
+      long q = p_info->q;
       const long *ap = &a.tbl[0][0];
       long *xp = &x.tbl[0][0];
       for (j = 0; j < n; j++) {
