@@ -889,13 +889,6 @@ void interpolate(zz_pX& f, const vec_zz_p& a, const vec_zz_p& b)
    f.rep = res;
 }
 
-NTL_vector_impl(zz_pX,vec_zz_pX)
-
-NTL_eq_vector_impl(zz_pX,vec_zz_pX)
-
-NTL_io_vector_impl(zz_pX,vec_zz_pX)
-
-
 
    
 void InnerProduct(zz_pX& x, const vec_zz_p& v, long low, long high, 
@@ -997,7 +990,7 @@ void build(zz_pXArgument& A, const zz_pX& h, const zz_pXModulus& F, long m)
 
 
 
-long zz_pXArgBound = 0;
+NTL_THREAD_LOCAL long zz_pXArgBound = 0;
 
 
 void CompMod(zz_pX& x, const zz_pX& g, const zz_pX& h, const zz_pXModulus& F)
@@ -1624,6 +1617,8 @@ void TraceMod(zz_p& x, const zz_pX& a, const zz_pXModulus& F)
    if (deg(a) >= n)
       Error("trace: bad args");
 
+   // FIXME: thread-safe-imple: need mutex here
+
    if (F.tracevec.length() == 0) 
       ComputeTraceVec(F);
 
@@ -1684,9 +1679,9 @@ void PlainResultant(zz_p& rres, const zz_pX& a, const zz_pX& b)
             break;
          }
       }
-
-      rres = res;
    }
+
+   rres = res;
 }
 
 
