@@ -674,7 +674,7 @@ long ProbIrredTest(const zz_pX& f, long iter)
    return !IsX(s);
 }
 
-long zz_pX_BlockingFactor = 10;
+NTL_THREAD_LOCAL long zz_pX_BlockingFactor = 10;
 
 void DDF(vec_pair_zz_pX_long& factors, const zz_pX& ff, const zz_pX& hh, 
          long verbose)
@@ -1531,12 +1531,11 @@ void BuildRandomIrred(zz_pX& f, const zz_pX& g)
 
 /************* NEW DDF ****************/
 
-long zz_pX_GCDTableSize = 4;
-
-static vec_zz_pX *BabyStepFile = 0;
-static vec_zz_pX *GiantStepFile = 0;
-static zz_pXArgument *HHH = 0;
-static long OldN = 0;
+NTL_THREAD_LOCAL long zz_pX_GCDTableSize = 4;
+NTL_THREAD_LOCAL static vec_zz_pX *BabyStepFile = 0;
+NTL_THREAD_LOCAL static vec_zz_pX *GiantStepFile = 0;
+NTL_THREAD_LOCAL static zz_pXArgument *HHH = 0;
+NTL_THREAD_LOCAL static long OldN = 0;
 
 
 static
@@ -1920,7 +1919,7 @@ void NewDDF(vec_pair_zz_pX_long& factors,
 
    if (deg(f) == 1) {
       factors.SetLength(0);
-      append(factors, cons(f, 1));
+      append(factors, cons(f, 1L));
       return;
    }
 
