@@ -1,25 +1,12 @@
 
-
 #include <NTL/vec_GF2E.h>
 
-#include <NTL/new.h>
 
 NTL_START_IMPL
 
-
-
-
-
-
-void BlockConstruct(GF2E* x, long n)
+static
+void BasicBlockConstruct(GF2E* x, long n, long d)
 {
-   if (n <= 0) return; 
-
-   if (!GF2EInfo)
-      Error("GF2E constructor called while modulus undefined");
-
-   long d = GF2E::WordLength();
-
    long m, j;
  
    long i = 0;
@@ -33,6 +20,42 @@ void BlockConstruct(GF2E* x, long n)
 }
 
 
+void BlockConstruct(GF2E* x, long n)
+{
+   if (n <= 0) return; 
+
+   if (!GF2EInfo)
+      Error("GF2E constructor called while modulus undefined");
+
+   long d = GF2E::WordLength();
+   BasicBlockConstruct(x, n, d);
+}
+
+void BlockConstructFromVec(GF2E* x, long n, const GF2E* y)
+{
+   if (n <= 0) return;
+
+   long d = y->_GF2E__rep.xrep.MaxLength();
+   BasicBlockConstruct(x, n, d);
+
+   long i;
+   for (i = 0; i < n; i++) x[i] = y[i];
+}
+
+void BlockConstructFromObj(GF2E* x, long n, const GF2E& y)
+{
+   if (n <= 0) return;
+
+   long d = y._GF2E__rep.xrep.MaxLength();
+   BasicBlockConstruct(x, n, d);
+
+   long i;
+   for (i = 0; i < n; i++) x[i] = y;
+}
+
+
+
+
 void BlockDestroy(GF2E* x, long n)
 {
    if (n <= 0) return;
@@ -48,13 +71,6 @@ void BlockDestroy(GF2E* x, long n)
 
 
 
-NTL_vector_impl_plain(GF2E,vec_GF2E)
-
-NTL_io_vector_impl(GF2E,vec_GF2E)
-
-NTL_eq_vector_impl(GF2E,vec_GF2E)
-
-
 void InnerProduct(GF2E& x, const vec_GF2E& a, const vec_GF2E& b)
 {
    long n = min(a.length(), b.length());
