@@ -1,17 +1,9 @@
 
 #include <NTL/vec_ZZ_pE.h>
 
-#include <NTL/new.h>
 
 NTL_START_IMPL
 
-NTL_vector_impl(ZZ_pE,vec_ZZ_pE)
-
-NTL_io_vector_impl(ZZ_pE,vec_ZZ_pE)
-
-NTL_eq_vector_impl(ZZ_pE,vec_ZZ_pE)
-
-
 void InnerProduct(ZZ_pE& x, const vec_ZZ_pE& a, const vec_ZZ_pE& b)
 {
    long n = min(a.length(), b.length());
