@@ -1,16 +1,8 @@
 
 #include <NTL/vec_lzz_p.h>
 
-#include <NTL/new.h>
-
 NTL_START_IMPL
 
-NTL_vector_impl(zz_p,vec_zz_p)
-
-NTL_io_vector_impl(zz_p,vec_zz_p)
-
-NTL_eq_vector_impl(zz_p,vec_zz_p)
-
 void conv(vec_zz_p& x, const vec_ZZ& a)
 {
    long i, n;
@@ -143,7 +135,7 @@ void mul(vec_zz_p& x, const vec_zz_p& a, zz_p b)
    if (n <= 1) {
 
       for (i = 0; i < n; i++)
-	 mul(x[i], a[i], b);
+         mul(x[i], a[i], b);
 
    }
    else {
